      SUBROUTINE MAID_G1G2(W,Q2,G1,G2,IPART)
*
* May 6, 2003, Seonho Choi
*
* Returns G1 and G2 from MAID calculation at given x and Q2
*
* IPART = 1 for the proton
*         2 for the neutron
*
* W in MeV
* Q2 in MeV**2
*
* G1,G2 are dimensionless (small g1/g2)
*
      PARAMETER (ALPHA=1.0/137.035989,PI=3.141592653589793238)      
      PARAMETER (DEG2RAD=0.01745329252,XMP=938.27231,XMN=939.56533,
     &		 XMPI=140.0)

      INTEGER IFIRST
      REAL XMASS(2),CONST(2)
      DATA IFIRST/0/
      DATA XMASS/938.27231,939.56533/
      SAVE IFIRST,XMASS,CONST

      IF (IFIRST.EQ.0) THEN
	 print *, 'reading MAID data (bitte haben sie etwas gedult).....'
	 print *
         CALL READ_DATA
         CONST(1) = 4.0*PI**2*ALPHA/XMP*0.389379292E9
         CONST(2) = 4.0*PI**2*ALPHA/XMN*0.389379292E9
         IFIRST = 1
      ENDIF
      
      IF (IPART.EQ.1) THEN ! proton
         CALL MAID_SIG(W,Q2,4,1,SIGLT1)
         CALL MAID_SIG(W,Q2,4,3,SIGLT2)
         CALL MAID_SIG(W,Q2,5,1,SIGTT1)
         CALL MAID_SIG(W,Q2,5,3,SIGTT2)
         SIG_LT = SIGLT1 + SIGLT2
         SIG_TT = SIGTT1 + SIGTT2
      ELSE ! neutron
         CALL MAID_SIG(W,Q2,4,2,SIGLT1)
         CALL MAID_SIG(W,Q2,4,4,SIGLT2)
         CALL MAID_SIG(W,Q2,5,2,SIGTT1)
         CALL MAID_SIG(W,Q2,5,4,SIGTT2)
         SIG_LT = SIGLT1 + SIGLT2
         SIG_TT = SIGTT1 + SIGTT2
      ENDIF
      XNU = (W**2 - XMASS(IPART)**2 + Q2)/(2.0*XMASS(IPART))
      COEF = SQRT(Q2)/XNU
      DENOM = 1.0 + COEF**2
      XK = (W**2 - XMASS(IPART)**2)/(2.0*XMASS(IPART))
      G1 = -XK/(CONST(IPART)*DENOM)*(SIG_TT+COEF*SIG_LT)
      G2 = XK/(CONST(IPART)*DENOM)*(SIG_TT-SIG_LT/COEF)
      RETURN
      END

      SUBROUTINE MAID_SIG(W_ARG,Q2_ARG,IFLAG,ICHAN,SIG_RET)

      INCLUDE 'maid.h'
      DIMENSION SIG_TEMP(10)
*
* Returns Sigma_TT from MAID at a given (W,Q2).
* W and Q2 should be in MeV
*
* The return cross seciton will be in mcb (micro-barn)
*
* IFLAG = 1 : T
*         2 : L
*         3 : LT
*         4 : LT'
*         5 : TT'
*         6 : L0
*         7 : LT0
*         8 : LT0'
*

*
* Range Check
*
      IF (W_ARG.LT.1080.0.OR.W_ARG.GT.2000.0.OR.Q2_ARG.LT.1.0E4.OR.
     &     Q2_ARG.GT.5.0E6) THEN
         SIG_RET = 0.0
         RETURN
      ENDIF
*
* Find I and J such that
* WG(I) <= W < WG(I+1) and
* Q2G(J) <= Q2 < Q2G(I+1)
*
      I = INT(0.2*W_ARG) - 215
      J = INT(1.0E-4*Q2_ARG)

*
* For a decent fitting, I need 4 points, 2 before and another 2 after.
*
      IMIN = I - 1
      IMAX = I + 2
      JMIN = J - 1
      JMAX = J + 2
*
* Check for the edges
*
      IF (IMIN.LT.1) THEN
         IMIN = 1
         IMAX = IMAX + (1-IMIN)
      ENDIF
      IF (IMAX.GT.185) THEN
         IMAX = 185
         IMIN = IMIN - (IMAX-185)
      ENDIF
      IF (JMIN.LT.1) THEN
         JMIN = 1
         JMAX = JMAX + (1-JMIN)
      ENDIF
      IF (JMAX.GT.500) THEN
         JMAX = 500
         JMIN = JMIN - (JMAX-500)
      ENDIF
      IF (Q2(J).EQ.Q2_ARG) THEN ! I need only 1 interpolation along W
         IF (I.GT.0.AND.W(I).EQ.W_ARG) THEN ! well, no need for interpolation at all.
            SIG_RET = SIG(I,J,ICHAN,IFLAG)
         ELSE
            CALL INTERPOL(W_ARG,W(IMIN),SIG(IMIN,J,ICHAN,IFLAG),SIG_RET)
         ENDIF
      ELSE
         DO K = JMIN,JMAX
            IF (I.GT.0.AND.W(I).EQ.W_ARG) THEN
               SIG_TEMP(K-JMIN+1) = SIG(I,K,ICHAN,IFLAG)
            ELSE
               CALL INTERPOL(W_ARG,W(IMIN),SIG(IMIN,K,ICHAN,IFLAG),
     &			     SIG_TEMP(K-JMIN+1))
            ENDIF
         ENDDO
         CALL INTERPOL(Q2_ARG,Q2(JMIN),SIG_TEMP,SIG_RET)
      ENDIF
      
      RETURN
      END

      SUBROUTINE INTERPOL(X,XP,YP,Y)
      DIMENSION XP(4),YP(4)
      
      CALL INTERPOL_PARA(X,XP,YP,Y1)
      CALL INTERPOL_PARA(X,XP(2),YP(2),Y2)
      Y = 0.5*(Y1+Y2)
C$$$      CALL INTERPOL_CUBIC(X,XP,YP,Y)
      RETURN
      END

      SUBROUTINE INTERPOL_PARA(X,XP,YP,Y)
      REAL XP,YP,X,Y
      DIMENSION XP(*),YP(*)

      REAL A,B,C,D1,D2,D3,D12,D13,D23

      D1 = X - XP(1)
      D2 = X - XP(2)
      D3 = X - XP(3)

      D12 = XP(1) - XP(2)
      D13 = XP(1) - XP(3)
      D23 = XP(2) - XP(3)

      A = D1*D2/(D13*D23)
      B = D2*D3/(D12*D13)
      C = D3*D1/(-D23*D12)

      Y = A*YP(3)+B*YP(1)+C*YP(2)
      RETURN
      END

      SUBROUTINE INTERPOL_CUBIC(XP,YP,X,Y)
      REAL XP,YP,X,Y
      DIMENSION XP(*),YP(*),C(4)

      D1 = X - XP(1)
      D2 = X - XP(2)
      D3 = X - XP(3)
      D4 = X - XP(4)

      D12 = XP(1) - XP(2)
      D13 = XP(1) - XP(3)
      D14 = XP(1) - XP(4)
      D23 = XP(2) - XP(3)
      D24 = XP(2) - XP(4)
      D34 = XP(3) - XP(4)

      C(1) = (D2*D3*D4)/(D12*D13*D14)
      C(2) = (D1*D3*D4)/(-D12*D23*D24)
      C(3) = (D1*D2*D4)/(D13*D23*D34)
      C(4) = (D1*D2*D3)/(-D14*D24*D34)

      Y = 0.0
      DO I = 1,4
         Y = Y + C(I)*YP(I)
      ENDDO
      RETURN
      END

      SUBROUTINE READ_DATA
      INCLUDE 'maid.h'
      OPEN(UNIT=10,FILE='maid_fine_full.dat',STATUS='OLD')
      DO IQ2 = 1,NQ2
         DO ICH = 1,4
            READ(10,*) JUNK,Q2_GEV
            Q2(IQ2) = Q2_GEV*1.0E6
            DO IW = 1,185
               READ(10,*) W(IW),XNU,(SIG(IW,IQ2,ICH,J),J=1,8)
            ENDDO
         ENDDO
      ENDDO
      CLOSE(10)
      RETURN
      END
