C **********************************************************************
        SUBROUTINE BONNQ (space,q,u0,u2)
C  Deuteron wavefunction from Bonn NN potential model.
C  Energy-independent relativistic momentum space OBEPQ
C  space=1: momentum space (q in 1/fm)
C  space=2: coordinate space (q=r in fm)
C **********************************************************************
        implicit none
        INTEGER space
        REAL*8  q,r,u0,u2,alpha,m0,pi
C...Parameterisation
        INTEGER j,n
        PARAMETER (n=11)
        REAL*8  CQ(n),DQ(n),m(n)
	real*8 dQ1,dQ2,dQ3,cQ1
        LOGICAL init
c...Save to memory
        save init,pi,m

        data init /.FALSE./
C...Energy-independent relativistic momentum space OBEPQ [ Cj,Dj in fm^-1/2 ]
        DATA    CQ /0.88628672D0, -0.27591814D0, -0.11610727D0,
     &             -0.12975243D2,  0.77490155D2, -0.27298039D3,
     &              0.53402693D3, -0.56328069D3,  0.30214616D3,
     &             -0.64920925D2,  0.00000000D0/
        DATA    DQ /0.23237078D-1, -0.52115578D0, -0.57197401D0,
     &              0.27570246D1, -0.26157324D2,  0.84419883D2,
     &             -0.98308997D2,  0.38498490D2,  0.00000000D0,
     &              0.00000000D0,  0.00000000D0/


        IF (init) GO TO 999     !Data already read/coefficients initialised
C...Parameterisation - initialise coefficients..........................
 123    init = .TRUE.
C...Value of pi
        pi = 4.D0*DATAN(1.D0)
C...alpha, m0 in 1/fm
        alpha = 0.231609D0
        m0 = 0.9D0
C...Set masses (units of 1/fm)
        DO j=1,n
c          m(j) = alpha + FLOAT(j-1)*m0
          m(j) = alpha + dble(j-1)*m0
        ENDDO
C...Set last Cj coefficient
        DO j=1,n-1
          CQ(n) = CQ(n) - CQ(j)
        ENDDO
C...Set last 3 Dj coefficients
        dQ1 = 0.D0
        dQ2 = 0.D0
        dQ3 = 0.D0
        DO j=1,n-3
          dQ1 = dQ1 + DQ(j)/m(j)**2
          dQ2 = dQ2 + DQ(j)
          dQ3 = dQ3 + DQ(j)*m(j)**2
        ENDDO
        DQ(n-2) = m(n-2)**2 / (m(n)**2 - m(n-2)**2)
     &                      / (m(n-1)**2 - m(n-2)**2)
     &          * ( - m(n-1)**2 * m(n)**2 * dQ1
     &              + (m(n-1)**2 + m(n)**2) * dQ2
     &              - dQ3 )
        DQ(n-1) = m(n-1)**2 / (m(n-2)**2 - m(n-1)**2)
     &                      / (m(n)**2 - m(n-1)**2)
     &          * ( - m(n)**2 * m(n-2)**2 * dQ1
     &              + (m(n)**2 + m(n-2)**2) * dQ2
     &              - dQ3 )
        DQ(n)   = m(n)**2 / (m(n-1)**2 - m(n)**2)
     &                    / (m(n-2)**2 - m(n)**2)
     &          * ( - m(n-2)**2 * m(n-1)**2 * dQ1
     &              + (m(n-2)**2 + m(n-1)**2) * dQ2
     &              - dQ3 )

C...Check constraints on Cj,Dj
c        cQ1 = 0.D0
c        dQ1 = 0.D0
c        dQ2 = 0.D0
c        dQ3 = 0.D0
c        DO j=1,n
c          cQ1 = cQ1 + cQ(j)
c          dQ1 = dQ1 + dQ(j)/m(j)**2
c          dQ2 = dQ2 + dQ(j)
c          dQ3 = dQ3 + dQ(j)*m(j)**2
c        ENDDO
c        print *, 'ZERO SUM?-cQ1,dQ1,dQ2,dQ3=',cQ1,dQ1,dQ2,dQ3

C...Evaluate wavefunction...............................................
 999    u0 = 0.D0
        u2 = 0.D0
C...Momentum space
        IF (space.EQ.1) THEN
              DO j=1,n
                u0 = u0 +  CQ(j) / (q**2 + m(j)**2)
                u2 = u2 -  DQ(j) / (q**2 + m(j)**2)
              ENDDO
                u0 = DSQRT(2.D0/pi) * u0
                u2 = DSQRT(2.D0/pi) * u2
C...Coordinate space
        ELSE IF (space.EQ.2 .AND. q.GT.0.D0) THEN
          r = q
              DO j=1,n
                u0 = u0 + CQ(j) * DEXP(-m(j)*r)
                u2 = u2 + DQ(j) * DEXP(-m(j)*r)
     &                    * (1.D0 + 3.D0/(m(j)*r) + 3.D0/(m(j)*r)**2)
              ENDDO
        ENDIF
        RETURN
        END
C **********************************************************************
        SUBROUTINE BONNF (space,q,u0,u2)
C  Deuteron wavefunction from Bonn NN potential model.
C  Energy-dependent full model
C  space=1: momentum space (q in 1/fm)
C  space=2: coordinate space (q=r in fm)
C **********************************************************************
        implicit none
        INTEGER space
        REAL*8  q,r,u0,u2,alpha,m0,pi
C...Parameterisation
        INTEGER j,n
        PARAMETER (n=11)
        REAL*8  CF(n),DF(n),m(n)
      real*8 dF1,dF2,dF3,cF1
        LOGICAL init
c...Save to memory
        save init,pi,m

        data init /.FALSE./

C...Energy-dependent full model [ Cj,Dj in fm^-1/2 ]
        DATA    CF /0.90457337D0, -0.35058661D0, -0.17635927D0,
     &             -0.10418261D2,  0.45089439D2, -0.14861947D3,
     &              0.31779642D3, -0.37496518D3,  0.22560032D3,
     &             -0.54858290D2,  0.00000000D0/
        DATA    DF /0.24133026D-1, -0.64430531D0, 0.51093352D0,
     &             -0.54419065D1,  0.15872034D2, -0.14742981D2,
     &              0.44956539D1, -0.71152863D-1, 0.00000000D0,
     &              0.00000000D0,  0.00000000D0/


        IF (init) GO TO 999     !Data already read/coefficients initialised
C...Parameterisation - initialise coefficients..........................
 123    init = .TRUE.
C...Value of pi
        pi = 4.D0*DATAN(1.D0)
C...alpha, m0 in 1/fm
        alpha = 0.231609D0
        m0 = 0.9D0
C...Set masses (units of 1/fm)
        DO j=1,n
c          m(j) = alpha + FLOAT(j-1)*m0
          m(j) = alpha + dble(j-1)*m0
        ENDDO
C...Set last Cj coefficient
        DO j=1,n-1
          CF(n) = CF(n) - CF(j)
        ENDDO
C...Set last 3 Dj coefficients
        dF1 = 0.D0
        dF2 = 0.D0
        dF3 = 0.D0
        DO j=1,n-3
          dF1 = dF1 + DF(j)/m(j)**2
          dF2 = dF2 + DF(j)
          dF3 = dF3 + DF(j)*m(j)**2
        ENDDO
        DF(n-2) = m(n-2)**2 / (m(n)**2 - m(n-2)**2)
     &                      / (m(n-1)**2 - m(n-2)**2)
     &          * ( - m(n-1)**2 * m(n)**2 * dF1
     &              + (m(n-1)**2 + m(n)**2) * dF2
     &              - dF3 )
        DF(n-1) = m(n-1)**2 / (m(n-2)**2 - m(n-1)**2)
     &                      / (m(n)**2 - m(n-1)**2)
     &          * ( - m(n)**2 * m(n-2)**2 * dF1
     &              + (m(n)**2 + m(n-2)**2) * dF2
     &              - dF3 )
        DF(n)   = m(n)**2 / (m(n-1)**2 - m(n)**2)
     &                    / (m(n-2)**2 - m(n)**2)
     &          * ( - m(n-2)**2 * m(n-1)**2 * dF1
     &              + (m(n-2)**2 + m(n-1)**2) * dF2
     &              - dF3 )

C...Check constraints on Cj,Dj
c        cF1 = 0.D0
c        dF1 = 0.D0
c        dF2 = 0.D0
c        dF3 = 0.D0
c        DO j=1,n
c          cF1 = cF1 + cF(j)
c          dF1 = dF1 + dF(j)/m(j)**2
c          dF2 = dF2 + dF(j)
c          dF3 = dF3 + dF(j)*m(j)**2
c        ENDDO
c        print *, 'ZERO SUM?-cF1,dF1,dF2,dF3=',cF1,dF1,dF2,dF3

C...Evaluate wavefunction...............................................
 999    u0 = 0.D0
        u2 = 0.D0
C...Momentum space
        IF (space.EQ.1) THEN
              DO j=1,n
                u0 = u0 +  CF(j) / (q**2 + m(j)**2)
                u2 = u2 -  DF(j) / (q**2 + m(j)**2)
              ENDDO
                u0 = DSQRT(2.D0/pi) * u0
                u2 = DSQRT(2.D0/pi) * u2
C...Coordinate space
        ELSE IF (space.EQ.2 .AND. q.GT.0.D0) THEN
          r = q
              DO j=1,n
                u0 = u0 + CF(j) * DEXP(-m(j)*r)
                u2 = u2 + DF(j) * DEXP(-m(j)*r)
     &                    * (1.D0 + 3.D0/(m(j)*r) + 3.D0/(m(j)*r)**2)
              ENDDO
        ENDIF
        RETURN
        END

