c********************************************************************
c  The function cnv2 computes the nucleon
c  structure function convoluted with nuclear
c  spectral function S(e,p) = n(p) delta(e-e0+p^2/2m0).
c********************************************************************
      real*8 function cnv2(x0,qsq0,e0,m0,nsf0)
      implicit none
      include "globalpar.inc"
      integer nsf,nsf0,nreg,A,Z
      real*8 e0,m0,x,qsq,q2,x0,qsq0,ao,gam,trial,prec,
     &       e00,m00,q,q0,mstar,pstar,cstar,mx2,gam2
      real*8 SD2,feff2d
c...Externals:
      external SD2,rgn2d,feff2d
c...Commons:
      common/nucleus/ A,Z
      common/par2d1/ mstar,pstar,cstar
      common/par2d2/ x,qsq,q2,gam,e00,m00,nsf
c 
c...Copy into common blocks
      x=x0
      qsq=qsq0
      nsf=nsf0
      e00=e0
      m00=m0
c 
c...Parameters:
      q2 = qsq/hc2
      mx2=(mN+mpi)**2
c      mx2=mN**2
      q0=q2/(2*mN*x)
      q=dsqrt(q2+q0**2)
      gam=q/q0
      gam2=1.d0+mN/q0
c      pstar=((q0+mN+e0)**2 - q**2 - mx2)/(2*q)
c      ao=m0/(m0+q0+mN+e0)
c      mstar=ao*q
      pstar=mN*(1.d0-x*(1.d0+(mx2-mN2)/q2)+e0*gam2/mN)/gam
      mstar=m0*q/(m0+q0+mN)
c
      if (pstar.GT.0d0) then
         cstar=-1.d0
      elseif (pstar.EQ.0d0) then
         cstar=0.d0
      else
         cstar=dsqrt(-2.d0*pstar/mstar)
      endif
c...Estimate/set the integration precision 
      if (x.lt.1d0) then
c         trial=tmc(x,qsq,yb,mN2*hc2,nsf,ityp,kint,kord,syst)
c         trial=dabs(trial)
c         prec=acc*trial
         if (prec.LT.1d-6) prec=1d-6
      else
         prec=1d-6
      endif
c...Call 2D integration routine.
      cnv2 = 0.d0
      if (pstar.GT.-mstar/2d0) 
     &   cnv2 = SD2(prec,feff2d,rgn2d)
      cnv2 = cnv2/2.d0
*
      return
      end

c**********************************************
c...Subroutine for the integration region (needed by SD2)
c...takes into account the constraint (p+q)^2 >= Mx^2,
c...where Mx is the mass of the final state.
c**********************************************
      subroutine rgn2d(L,xn,a,b)
      implicit none
      include "globalpar.inc"
      real*8 xn(2),a(2),b(2),pmax,pmin,dum,dnm
      real*8 mstar,pstar,cstar
      integer L
      common/par2d1/ mstar,pstar,cstar
*
      if (L.eq.1) then
c...cos(theta) integration limits
         a(1) = cstar
         b(1) = 1.d0
      elseif (L.eq.2) then
c...momentum integration limits
         dum = mstar*xn(1)
         dnm = dum**2 + 2.d0*mstar*pstar
c debug:
c              if (dnm.lt.0d0) then
c          print *,'rgn2d::plimits:pstar,cstar=',pstar,cstar
c          print *,'rgn2d::plimits:dnm=',dnm
c              endif
c...next line is to avoid small but annoying negative numbers in sqrt
         if(dnm.lt.0d0.and.dabs(dnm).lt.tlrnc) dnm=0d0
         pmax = dum + dsqrt(dnm)
         pmax = dmin1(pmax,pcut)
         pmin = 0d0
         if (pstar.lt.0d0) then
            pmin = dum - dsqrt(dnm)
            pmin = dmin1(pmin,pcut)
         endif
         a(2) = dexp(-pmax)
         b(2) = dexp(-pmin)
      endif
      return
      end
c**********************************************
c...The integrand in the 2d convolution formula
c**********************************************
      real*8 function feff2d(u)
      implicit none
      include "globalpar.inc"
      integer nsf,A,Z
      real*8 u(2),psi0,psi2,ssf
      real*8 y,p,e,pz,t,psq,x1,x,qsq,q2,gam,e00,m00,
     &       fac1,fac2
Commons:
      common/nucleus/ A,Z
      common/par2d2/ x,qsq,q2,gam,e00,m00,nsf

      feff2d = 0.d0
* u(1) = cos(theta)
* u(2) = exp(-p)
      y = u(1)
      p = dlog( 1.d0/u(2) )
      e = e00 - p**2/(2.d0*m00)
* pz= p*cos(theta)
* t = p_perp**2
      pz = p*y
      t  = p**2*(1.d0 - y**2)
      psq= (mN+e)**2 - p**2
      x1 = x/(1.d0 + (e+gam*pz)/mN)
c  Deuteron momentum distribution
      call paris(1,p,psi0,psi2)     ! Paris NN potential model
cc      call bonnq(1,p,psi0,psi2)     ! Bonn NN potential model
        fac1 = (psi0**2 + psi2**2) * p**2
c Integrand: ssf and kinematical factors
      if (nsf.EQ.1) then
c The integrand for xgT
         fac2 = 1d0 !!(1.d0 - p**2/mN2/2d0)
c     &        * ( 1.d0 + (4.d0*psq+6.d0*t)*x1**2/q2 )/gam**2
     &        * (ssf(0,x1,qsq,1) + ssf(0,x1,qsq,2))
      elseif (nsf.EQ.2) then
c The integrand for xgL
         fac2 = 1d0 !!(1.d0 - p**2/mN2/2d0)
c     &        * ( 1.d0 + (4.d0*psq+6.d0*t)*x1**2/q2 )/gam**2
     &        * ssf(0,x1,qsq,1)
      else
         write(*,*) 'cnv2: nsf parameter is out of range, nsf=',nsf
         return
      endif
*
      feff2d = fac1*fac2 /u(2)
*
      return
      end
*******END of cnv2*******
