c********************************************
c  The function conv3d returns the nucleon
c  structure function convoluted with nuclear
c  spectral function S(A,Z,e,p).
c*******************************************
      real*8 function conv3d(x0,qsq0,yb0,bg0,e0,m0,
     +                       nsf0,ityp0,kint0,kord0,syst0)
      implicit none
#include "globalpar.inc"
      integer nsf,ityp,kint,kord
      integer nsf0,ityp0,kint0,kord0,nreg
      real syst0,syst
      real*8 e0,m0,x,qsq,yb,q2,x0,qsq0,yb0,ao,gam,trial,prec,
     &       e00,m00,q,q0,mstar,pstar,cstar,mx2,gam2,bg0,bg
      real*8 SDN,feff3d,tmc
c...Externals:
      external SDN
      external rgn3d1,rgn3d2,feff3d
c...Commons:
      common/par3d1/ e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
      common/par3d2/ x,qsq,yb,q2,gam,nsf,ityp,kint,kord,syst
c
c...  Copy into common blocks
c
      x=x0
      qsq=qsq0
      yb=yb0
      nsf=nsf0
      ityp=ityp0
      kint=kint0
      kord=kord0
      syst=syst0
      e00=e0
      m00=m0
      bg=bg0
c 
c...Parameters of the routine
      q2 = qsq/hc2
      mx2=(mN+mpi)**2
c      mx2=mN**2
      q0=q2/(2*mN*x)
      q=dsqrt(q2+q0**2)
      gam=q/q0
      gam2=1.d0+mN/q0
c      pstar=((q0+mN+e0)**2 - q**2 - mx2)/(2*q)
c      ao=m0/(m0+q0+mN+e0)
c      mstar=ao*q
      pstar=mN*(1.d0-x*(1.d0+(mx2-mN2)/q2)+e0*gam2/mN)/gam
      mstar=m0*q/(m0+q0+mN)
c
      if (pstar.GT.0d0) then
         cstar=-1.d0
      elseif (pstar.EQ.0d0) then
         cstar=0.d0
      else
         cstar=dsqrt(-2.d0*pstar/mstar)
      endif
c
c...Estimate/set integration precision
c
      if (x.lt.1d0) then
         trial=tmc(x,qsq,yb,mN2*hc2,nsf,ityp,kint,kord,syst)
         trial=dabs(trial)
         prec=acc*trial
         if (prec.LT.1d-6) prec=1d-6
      else
         prec=1d-6
      endif
c
c  Call 3D integration routine.
c  The factor scalefac is introduced to avoid small numbers
c  in the integrals.
c
      conv3d = 0.d0
      if (pstar.GT.-mstar/2d0) then
         conv3d = SDN(3,prec,feff3d,rgn3d1)+
     +            SDN(3,prec,feff3d,rgn3d2)
      endif
      conv3d = conv3d/(4d0*pi)/scalefac

      return
      end
c**********************************************
c  Subroutines for integration regions:
c**********************************************
      subroutine rgn3d1(L,xn,a,b)
      implicit none
#include "globalpar.inc"
      real*8 xn(3),a(3),b(3),pmax,pmin,dum,eth,ex,emax,pr,dnm
      real*8 e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
      integer L
      common/par3d1/ e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
*
      if (L.eq.1) then
c cos(theta) integration limits
         a(1)=cstar
         b(1)=1.d0
      elseif (L.eq.2) then
c momentum integration limits
         dum=mstar*xn(1)
         dnm=dum**2 + 2.d0*mstar*pstar
c debug:
c              if (dnm.lt.0d0) then
c          print *,'rgn3d1::plimits:pstar,cstar=',pstar,cstar
c          print *,'rgn3d1::plimits:dnm=',dnm
c              endif
c the next line is to avoid small but annoying negative numbers in sqrt
         if(dnm.lt.0d0.and.dabs(dnm).lt.tlrnc) dnm=0d0
         pmax=dum + dsqrt(dnm)
         pmax=dmin1(pmax,pcut)
         pmin=0d0
         if (pstar.lt.0d0) then
            pmin=dum - dsqrt(dnm)
            pmin=dmin1(pmin,pcut)
         endif
c debug:
c           print *,'pmin,pmax=',pmin,pmax
         a(2)=dexp(-pmax)
         b(2)=dexp(-pmin)
      elseif (L.eq.3) then
c energy integration limits
         pr=dlog(1.d0/xn(2))
         eth=e00 - pr**2/(2.d0*m00)
         dum=mx2 + pr**2 - 2.d0*pr*q*xn(1) + q**2
c debug:
c                  if (dum.lt.0d0) then
c                      print *,'rgn3d1::elimits:dum=',dum
c                  endif
         ex=dsqrt(dum)
         emax=mN+eth+q0 - ex
         a(3)=0d0
         b(3)=4.d0*pr/mN
         if (b(3).gt.emax/2.d0)  b(3)=emax/2.d0
c debug:
c          if (emax.lt.0d0) then
c           print *,'rgn3d1::a(3),b(3)=',a(3),b(3)
c          endif
      endif
c 
      return
      end
c**********************************************
      subroutine rgn3d2(L,xn,a,b)
      implicit none
#include "globalpar.inc"
      real*8 xn(3),a(3),b(3),pmax,pmin,dum,eth,ex,emax,pr,dnm
      real*8 e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
      integer L
      common/par3d1/ e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
*
      if (L.eq.1) then
c cos(theta) integration limits
         a(1)=cstar
         b(1)=1.d0
      elseif (L.eq.2) then
c momentum integration limits
         dum=mstar*xn(1)
         dnm=dum**2 + 2.d0*mstar*pstar
c debug:
c                if (dnm.lt.0d0) then
c          print *,'rgn3d2::plimits:pstar,cstar=',pstar,cstar
c          print *,'rgn3d2::plimits:dnm=',dnm
c                endif
c the next line is to avoid small but annoying negative numbers in sqrt
         if(dnm.lt.0d0.and.dabs(dnm).lt.tlrnc) dnm=0d0
         pmax=dum + dsqrt(dnm)
         pmax=dmin1(pmax,pcut)
         pmin=0d0
         if (pstar.lt.0d0) then
            pmin=dum - dsqrt(dnm)
            pmin=dmin1(pmin,pcut)
         endif
         a(2)=dexp(-pmax)
         b(2)=dexp(-pmin)
      elseif (L.eq.3) then
c energy integration limits
         pr=dlog(1.d0/xn(2))
         eth=e00 - pr**2/(2d0*m00)
         dum=mx2 + pr**2 - 2.d0*pr*q*xn(1) + q**2
c debug:
c                  if (dum.lt.0d0) then
c         print *,'rgn3d2::elimits:dum=',dum
c                  endif
         ex=dsqrt(dum)
         emax=mN+eth+q0 - ex
         a(3)=4.d0*pr/mN
            if (a(3).gt.emax/2.d0)  a(3)=emax/2.d0
         b(3)=emax
c debug:
c            if (emax.lt.0d0) then
c              print*,'rgn3d2::a(3),b(3)=',a(3),b(3)
c            endif
      endif
c 
      return
      end
c**********************************************
c  The integrand in 3D convolution formula
c**********************************************
      real*8 function feff3d(u)
      implicit none
#include "globalpar.inc"
      integer nsf,ityp,kint,kord,A,Z
      real syst
      real*8 u(3)
      real*8 y,p,e,eprim,eth,pz,t,psq,x1,x,qsq,yb,q2,gam,
     &       fac1,fac2,n0,n1,tmc,flt,spfn1,
     &       e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
c...Externals:
c      external n0,n1,f2,f1
c...Commons:
      common/nucleus/ A,Z
      common/par3d1/ e00,m00,q,q0,mstar,pstar,cstar,mx2,bg
      common/par3d2/ x,qsq,yb,q2,gam,nsf,ityp,kint,kord,syst
*
      feff3d = 0d0
c 
c  u(1)= cos(theta)
c  u(2)= exp(-p)
c  u(3)= eprim
      y = u(1)
      p = dlog( 1.d0/u(2) )
      eprim = u(3)
      eth = e00 - p**2/(2.d0*m00)
      e = eth - eprim
c 
c pz=p*cos(theta)
c t=p_perp**2
      pz = p*y
      t = p**2*(1.d0 - y**2)
      psq = (mN+e)**2 - p**2
      x1 = x/(1.d0 + (e+gam*pz)/mN)
c 
c...Nuclear spectral function spfn1 from nuclspec package
      fac1 = spfn1(A,Z,bg,eprim,p)
*
      if (nsf.EQ.2) then
c...The integrand for F2
         fac2 = (1.d0 + pz*gam/mN)
     &        * ( 1.d0 + (4.d0*psq+6.d0*t)*x1**2/q2 )/gam**2
     &        * tmc(x1,qsq,yb,psq*hc2,2,ityp,kint,kord,syst)
      elseif (nsf.EQ.1) then
c...The integrand for FT = 2*x*F1
         fac2 = (1.d0 + pz*gam/mN)
     &        * (2.d0*x1**2*t/q2
     &        * tmc(x1,qsq,yb,psq*hc2,2,ityp,kint,kord,syst)
     &        + tmc(x1,qsq,yb,psq*hc2,1,ityp,kint,kord,syst) )
      elseif (nsf.EQ.3) then
c...The integrand for xF3
         fac2 = (1.d0 + pz/gam/mN)
     &        * tmc(x1,qsq,yb,psq*hc2,3,ityp,kint,kord,syst)
      else
         write(*,*) 'nsf parameter is out of range, nsf=',nsf
         return
      endif
*
      feff3d = fac1*fac2*scalefac /u(2)
*
      return
      end
*******END of conv3d*******
