C *******************************************************************
	PROGRAM HE
C
C     TITEL:       SUMREG (-> HE)
C     VERSION:     11.MAI.1994                                      *
C     REVISED:                                                      *
C                                                                   *
C     SUBROUTINES:  nag-routines                                    *
C                                                                   *
C     Description:  This routine calculates the two dimensional     *
C                   integral over the spectral function             *
C                   \int d3p_N dE S(p_N,E)
C               In addition:                                        *
C               This routine is intended to give precise hints on   *
C               how the spectral function is normalized and saved.  *
C               The spectral function is based on the three-body    *
C               wave function calculated on a finite grid with      *
C               40*30 mesh points for the (p,q=p_N)-momenta. It is  *
C               again calculated on a finite grid with (30*30) mesh *
C               points for the momentum/energy mesh (p_N,E).        *
C                                                                   * 
C 	Energy in MeV
C	Momentum in 1/fm
C                                                                   *
C     Compile with: %f77 -r8 sumreg.f                               * 
C                   %f77 -o prog sumreg.o -lnag                     *
C                                                                   *
C
C  Modified for JLab: March 2003
C  Updated: June 2003
C
C  With IMSL routines:
C  compile > use imsl
C          > f77 -o objectfile file.f
C                -R/site/vni/lib/lib.solaris
C                -L/site/vni/lib/lib.solaris -limsl -lsocket -lnsl
C
C *******************************************************************
	IMPLICIT NONE
	INTEGER	NDE,NDE1,NDP,NDQ,NERG,IE,NQ,IQ,JQ,I,J, IEN,NERG,
     &		ne,ie, np,ip, npz,ipz, npT,ipT, nx,ix, ny,iy, iq2
	PARAMETER (NDE=32,NDE1=NDE-1,NDP=40,NDQ=30, nx=100)
	REAL*4	ENER(NDE),QMESH(NDQ),QWEIGH(NDQ),
     &		R0T0(NDP,NDQ),R0T1(NDP,NDQ),
     &		R1T0(NDP,NDQ),R1T1(NDP,NDQ),
     &		R2T0(NDP,NDQ),R2T1(NDP,NDQ),
     &		R0T0ar(NDE1,NDQ),R0T1ar(NDE1,NDQ),
     &		R1T0ar(NDE1,NDQ),R1T1ar(NDE1,NDQ),
     &		R2T0ar(NDE1,NDQ),R2T1ar(NDE1,NDQ),
     &		TWO0T0(NDQ),TWO1T0(NDQ),TWO2T0(NDQ)
	REAL*4	F0T0,F0T1,F1T0,F1T1,F2T0,F2T1,
     &		F0T0p2,F0T1p2,F1T0p2,F1T1p2,F2T0p2,F2T1p2,
     &		F0T0E,F0T1E,F1T0E,F1T1E
	REAL*4	E_ARRAY(29),
     &		FEar0T0(29),FEar0T1(29),
     &		FEar1T0(29),FEar1T1(29),
     &		FEar2T0(29),FEar2T1(29),
     &		ans0T0,ans0T1,ans1T0,ans1T1,ans2T0,ans2T1,
     &		ans0T0E,ans0T1E,ans1T0E,ans1T1E,
     &		P_ARRAY(30),
     &		F0T0ar(30),F0T1ar(30),
     &		F1T0ar(30),F1T1ar(30),
     &		F2T0ar(30),F2T1ar(30),
     &		F0T0Ear(30),F0T1Ear(30),
     &		F1T0Ear(30),F1T1Ear(30),
     &		yy0T0,yy0T1,yy1T0,yy1T1,yy2T0,yy2T1,
     &		pp0T0,pp0T1,pp1T0,pp1T1,pp2T0,pp2T1,
     &		pp0T0E,pp0T1E,pp1T0E,pp1T1E
	REAL*4	QDVAL
	REAL*4	e,emin,emax,eint, eps, p,p2,pmax,pint,
     &		pz,pz2,pzmin,pzmax,pzint, pT,pT2,pTmin,pTmax,pTint,px2
	REAL*4	N0p,N0n,N1p,N1n,N2p,N2n,
     &		T0p,T0n,T1p,T1n,T2p,T2n, E0p,E0n,E1p,E1n,
     &		norm0,norm1
	REAL*4	pi,hc,hc2, mN,mN2,E_B,M2,M3,E_d,ss,qvec,a,b
	REAL*8	dxg1p,dxg1n,dxg2p,dxg2n
	REAL*4	Au,Bu,Cu,Du,Ad,Bd,Cd,Dd, xDu,xDd
	REAL*4	x,xmin,xmax,xint,
     &		xg1p(nx),xg1n(nx),xg2p(nx),xg2n(nx),G1,G2,
     &		y,yint,
     &		z,zg1p,zg1n,zg2p,zg2n
	REAL*4	Gam1n,Gam1p,Gam1A_n,Gam1A_p,
     &		Gam2n,Gam2p,Gam2A_n,Gam2A_p,
     &		d2n,d2p,d2A_n,d2A_p
	REAL*4	f0p,f0n,f1p,f1n,f2p,f2n,
     &		prl_p,prl_n,prp_p,prp_n,
     &		int1p,int2p,int1n,int2n,
     &		pp0_z,nn0_z,pp1_z,nn1_z,pp2_z,nn2_z,
     &		pp0_T,nn0_T,pp1_T,nn1_T,pp2_T,nn2_T,
     &		pp0,nn0,pp1,nn1,pp2,nn2, eps2,z2,
     &		p0,n0,p1,n1,p2,n2,
     &		Ep0ar(29),En0ar(29),
     &		Ep1ar(29),En1ar(29),
     &		Ep2ar(29),En2ar(29),
     &		P0T0ar(30),P0T1ar(30),
     &		P1T0ar(30),P1T1ar(30),
     &		P2T0ar(30),P2T1ar(30)
	REAL*4	xN(nx),xg1A_p(nx),xg1A_n(nx),xg2A_p(nx),xg2A_n(nx)
	REAL*4	Q2,Q2gev,W,gam,gam2,eu,ed, q2ar(7)
	data    q2ar /0.10, 0.26, 0.42, 0.58, 0.74, 0.90, 2.0/      

	pi = 4*ATAN(1.0)
	hc = 193.0		! MeV.fm
	hc2 = hc**2		! (MeV.fm)^2
	mN = 939.0/hc		! MeV => 1/fm
	mN2 = mN**2
	E_B = -7.72		! He3 binding energy (MeV)
	M3 = 3*mN + E_B/hc	! mass of A=3 nucleus (1/fm)
	E_d = -2.224575		! d binding energy (MeV)


	do iq2=7,7

	Q2gev = q2ar(iq2)	! GeV^2
	Q2 = (Q2gev*1.E6)/hc2	! => 1/fm^2

c	open(88,FILE='he.out',STATUS='UNKNOWN',FORM='FORMATTED')

	eu = 4.0/9.0		! quark charges squared
	ed = 1.0/9.0

	OPEN (UNIT=1,FILE='/u/home/wmelnitc/Work/EMC/Tri/NR/spfn.dat')

C-----input of the spectral function:
C
C     The spectral function is given for the pair-isospin 
C     T=0 and T=1
C     The two-body breakup is given in the first columne, i.e.,
C     R0T0(1,J) etc.
C     R0T0(2..NERG,J) contains the three-body breakup.
C
C     The normalization is chosen such that the proton and 
C     neutron contribution is given by:
C
C      R0P = (F0T0 + F0T1)*4.*PI/3.
C      R0N = 2.*F0T1*4.*PI/3.
C
C     with R0P = 2/3
C     and  R0N = 1/3
C
C     Since the spectral function is given on a finite mesh
C     the sumrule is not exactly fullfilled !
C     The mesh was chosen for a particular kinematic situation.
C
C     The momentum mesh is given for Gaussian points.  

	READ(1,'(2I3)')NERG,NQ
	READ(1,'(6E13.6)')(ENER(IE),IE=1,NERG)
	READ(1,'(6E13.6)')(QMESH(IQ),IQ=1,NQ),(QWEIGH(IQ),IQ=1,NQ)
	READ(1,'(6E13.6)')((R0T0(I,J),R1T0(I,J),R2T0(I,J),J=1,NQ),I=1,NERG)
	READ(1,'(6E13.6)')((R0T1(I,J),R1T1(I,J),R2T1(I,J),J=1,NQ),I=1,NERG)
	CLOSE (1)


C...Normalizations.......................................................
	print *, 'reading spectral function...'
C...Calculate energy integral at given momentum (1/fm)
	DO JQ=1,NQ

C...Momentum variable array (1/fm)
	  P_ARRAY(JQ) = QMESH(JQ)

C...Two-body contribution (for isoscalar channel only, of course)
	  TWO0T0(JQ) = R0T0(1,JQ)
	  TWO1T0(JQ) = R1T0(1,JQ)
	  TWO2T0(JQ) = R2T0(1,JQ)

C...Set up energy array (MeV)
	  DO IEN=2,NERG

	    E_ARRAY(IEN-1) = ENER(IEN)

	    FEar0T0(IEN-1) = R0T0(IEN,JQ)
	    FEar0T1(IEN-1) = R0T1(IEN,JQ)
	    FEar1T0(IEN-1) = R1T0(IEN,JQ)
	    FEar1T1(IEN-1) = R1T1(IEN,JQ)
	    FEar2T0(IEN-1) = R2T0(IEN,JQ)
	    FEar2T1(IEN-1) = R2T1(IEN,JQ)

	  ENDDO

	  emin = 0.01
	  emax = 150.0	! 68.0
	  ne = 10000
	  eint = (emax-emin)/ne
	  ans0T0 = 0.0
	  ans0T1 = 0.0
	  ans1T0 = 0.0
	  ans1T1 = 0.0
	  ans2T0 = 0.0
	  ans2T1 = 0.0
	  ans0T0E = 0.0
	  ans0T1E = 0.0
	  ans1T0E = 0.0
	  ans1T1E = 0.0
	  ie = 0

C...Integrate over energy (NB: E_{sauer} = -epsilon - p^2/4mN)
	  DO e=emin,emax,eint

	    yy0T0 = QDVAL(e,NERG-1,E_ARRAY,FEar0T0,.FALSE.)
	    yy0T1 = QDVAL(e,NERG-1,E_ARRAY,FEar0T1,.FALSE.)
	    yy1T0 = QDVAL(e,NERG-1,E_ARRAY,FEar1T0,.FALSE.)
	    yy1T1 = QDVAL(e,NERG-1,E_ARRAY,FEar1T1,.FALSE.)
	    yy2T0 = QDVAL(e,NERG-1,E_ARRAY,FEar2T0,.FALSE.)
	    yy2T1 = QDVAL(e,NERG-1,E_ARRAY,FEar2T1,.FALSE.)

	    IF (ie.EQ.0) THEN
	      ans0T0 = ans0T0 + yy0T0
	      ans0T1 = ans0T1 + yy0T1
	      ans1T0 = ans1T0 + yy1T0
	      ans1T1 = ans1T1 + yy1T1
	      ans2T0 = ans2T0 + yy2T0
	      ans2T1 = ans2T1 + yy2T1
	      ans0T0E = ans0T0E + yy0T0*(e/hc)/mN	! e in MeV
	      ans0T1E = ans0T1E + yy0T1*(e/hc)/mN	! mN in 1/fm!!
	      ans1T0E = ans1T0E + yy1T0*(e/hc)/mN
	      ans1T1E = ans1T1E + yy1T1*(e/hc)/mN
	    ELSE IF (ie/2*2.NE.ie) THEN
	      ans0T0 = ans0T0 + 4*yy0T0
	      ans0T1 = ans0T1 + 4*yy0T1
	      ans1T0 = ans1T0 + 4*yy1T0
	      ans1T1 = ans1T1 + 4*yy1T1
	      ans2T0 = ans2T0 + 4*yy2T0
	      ans2T1 = ans2T1 + 4*yy2T1
	      ans0T0E = ans0T0E + 4*yy0T0*(e/hc)/mN
	      ans0T1E = ans0T1E + 4*yy0T1*(e/hc)/mN
	      ans1T0E = ans1T0E + 4*yy1T0*(e/hc)/mN
	      ans1T1E = ans1T1E + 4*yy1T1*(e/hc)/mN
	    ELSE IF (ie/2*2.EQ.ie) THEN
	      ans0T0 = ans0T0 + 2*yy0T0
	      ans0T1 = ans0T1 + 2*yy0T1
	      ans1T0 = ans1T0 + 2*yy1T0
	      ans1T1 = ans1T1 + 2*yy1T1
	      ans2T0 = ans2T0 + 2*yy2T0
	      ans2T1 = ans2T1 + 2*yy2T1
	      ans0T0E = ans0T0E + 2*yy0T0*(e/hc)/mN
	      ans0T1E = ans0T1E + 2*yy0T1*(e/hc)/mN
	      ans1T0E = ans1T0E + 2*yy1T0*(e/hc)/mN
	      ans1T1E = ans1T1E + 2*yy1T1*(e/hc)/mN
	    ENDIF
	    ie = ie + 1

	  ENDDO
	  ans0T0 = (eint/3)*ans0T0
	  ans0T1 = (eint/3)*ans0T1
	  ans1T0 = (eint/3)*ans1T0
	  ans1T1 = (eint/3)*ans1T1
	  ans2T0 = (eint/3)*ans2T0
	  ans2T1 = (eint/3)*ans2T1
	  ans0T0E = (eint/3)*ans0T0E
	  ans0T1E = (eint/3)*ans0T1E
	  ans1T0E = (eint/3)*ans1T0E
	  ans1T1E = (eint/3)*ans1T1E

c	print *, 'JQ,Q,ans0T0=',jq,qmesh(jq),ans0T0

	  F0T0ar(JQ) = (ans0T0 + TWO0T0(JQ)) * QMESH(JQ)**2
	  F0T1ar(JQ) =  ans0T1 * QMESH(JQ)**2
	  F1T0ar(JQ) = (ans1T0 + TWO1T0(JQ)) * QMESH(JQ)**2
	  F1T1ar(JQ) =  ans1T1 * QMESH(JQ)**2
! 1T0 => -0.484089
!        ans = -3.21168E-02
!	 TWO = -0.451974
! 1T1 =>  0.436272
!
	  F2T0ar(JQ) = (ans2T0 + TWO2T0(JQ)) * QMESH(JQ)**2
	  F2T1ar(JQ) =  ans2T1 * QMESH(JQ)**2
! 2T0 => -0.260223
!        ans = -2.32713E-02
!	 TWO = -0.236951    
! 2T1 => 1.63898E-02
!
	  F0T0Ear(JQ) = (ans0T0E + TWO0T0(JQ)*(-E_B+E_d)/hc/mN)
     &		      * QMESH(JQ)**2
	  F0T1Ear(JQ) =  ans0T1E * QMESH(JQ)**2
	  F1T0Ear(JQ) = (ans1T0E + TWO1T0(JQ)*(-E_B+E_d)/hc/mN)
     &		      * QMESH(JQ)**2
	  F1T1Ear(JQ) =  ans1T1E * QMESH(JQ)**2

	ENDDO
c	print *, 'momentum grid set up... now integrate over p'


C...Arrays for interpolation
	DO IEN=2,NERG
	  DO JQ=1,NQ
	    R0T0ar(IEN-1,JQ) = R0T0(IEN,JQ)	! for use in two-dim.
	    R0T1ar(IEN-1,JQ) = R0T1(IEN,JQ)	! interpolation
	    R1T0ar(IEN-1,JQ) = R1T0(IEN,JQ)
	    R1T1ar(IEN-1,JQ) = R1T1(IEN,JQ)
	    R2T0ar(IEN-1,JQ) = R2T0(IEN,JQ)
	    R2T1ar(IEN-1,JQ) = R2T1(IEN,JQ)
	  ENDDO
	ENDDO


C...Momentum integration
	F0T0 = 0.0
	F0T1 = 0.0
	F1T0 = 0.0
	F1T1 = 0.0
	F2T0 = 0.0
	F2T1 = 0.0
	F0T0p2 = 0.0
	F0T1p2 = 0.0
	F1T0p2 = 0.0
	F1T1p2 = 0.0
	F2T0p2 = 0.0
	F2T1p2 = 0.0
	F0T0E = 0.0
	F0T1E = 0.0
	F1T0E = 0.0
	F1T1E = 0.0

	pmax = 6.5
	np = 10000
	pint = pmax/np
	ip = 1
	DO p=0,pmax,pint

	  p2 = p**2	  
	  pp0T0 = QDVAL(p,NQ,P_ARRAY,F0T0ar,.FALSE.)
	  pp0T1 = QDVAL(p,NQ,P_ARRAY,F0T1ar,.FALSE.)
	  pp1T0 = QDVAL(p,NQ,P_ARRAY,F1T0ar,.FALSE.)
	  pp1T1 = QDVAL(p,NQ,P_ARRAY,F1T1ar,.FALSE.)
	  pp2T0 = QDVAL(p,NQ,P_ARRAY,F2T0ar,.FALSE.)
	  pp2T1 = QDVAL(p,NQ,P_ARRAY,F2T1ar,.FALSE.)
	  pp0T0E = QDVAL(p,NQ,P_ARRAY,F1T0Ear,.FALSE.)
	  pp0T1E = QDVAL(p,NQ,P_ARRAY,F1T1Ear,.FALSE.)
	  pp1T0E = QDVAL(p,NQ,P_ARRAY,F1T0Ear,.FALSE.)
	  pp1T1E = QDVAL(p,NQ,P_ARRAY,F1T1Ear,.FALSE.)

	  IF (ip/2*2.NE.ip) THEN
	    F0T0 = F0T0 + 4*pp0T0
	    F0T1 = F0T1 + 4*pp0T1
	    F1T0 = F1T0 + 4*pp1T0
	    F1T1 = F1T1 + 4*pp1T1
	    F2T0 = F2T0 + 4*pp2T0
	    F2T1 = F2T1 + 4*pp2T1
	    F0T0p2 = F0T0p2 + 4*pp0T0*(p2/mN2)
	    F0T1p2 = F0T1p2 + 4*pp0T1*(p2/mN2)
	    F1T0p2 = F1T0p2 + 4*pp1T0*(p2/mN2)
	    F1T1p2 = F1T1p2 + 4*pp1T1*(p2/mN2)
	    F2T0p2 = F2T0p2 + 4*pp2T0*(p2/mN2)
	    F2T1p2 = F2T1p2 + 4*pp2T1*(p2/mN2)
	    F0T0E = F0T0E + 4*pp0T0E
	    F0T1E = F0T1E + 4*pp0T1E
	    F1T0E = F1T0E + 4*pp1T0E
	    F1T1E = F1T1E + 4*pp1T1E
	  ELSE IF (ip/2*2.EQ.ip) THEN
	    F0T0 = F0T0 + 2*pp0T0
	    F0T1 = F0T1 + 2*pp0T1
	    F1T0 = F1T0 + 2*pp1T0
	    F1T1 = F1T1 + 2*pp1T1
	    F2T0 = F2T0 + 2*pp2T0
	    F2T1 = F2T1 + 2*pp2T1
	    F0T0p2 = F0T0p2 + 2*pp0T0*(p2/mN2)
	    F0T1p2 = F0T1p2 + 2*pp0T1*(p2/mN2)
	    F1T0p2 = F1T0p2 + 2*pp1T0*(p2/mN2)
	    F1T1p2 = F1T1p2 + 2*pp1T1*(p2/mN2)
	    F2T0p2 = F2T0p2 + 2*pp2T0*(p2/mN2)  
	    F2T1p2 = F2T1p2 + 2*pp2T1*(p2/mN2)
	    F0T0E = F0T0E + 2*pp0T0E
	    F0T1E = F0T1E + 2*pp0T1E
	    F1T0E = F1T0E + 2*pp1T0E
	    F1T1E = F1T1E + 2*pp1T1E
	  ENDIF

	  ip = ip + 1

	ENDDO
	F0T0 = (pint/3)*F0T0
	F0T1 = (pint/3)*F0T1
	F1T0 = (pint/3)*F1T0
	F1T1 = (pint/3)*F1T1
	F2T0 = (pint/3)*F2T0
	F2T1 = (pint/3)*F2T1
	F0T0p2 = (pint/3)*F0T0p2
	F0T1p2 = (pint/3)*F0T1p2
	F1T0p2 = (pint/3)*F1T0p2
	F1T1p2 = (pint/3)*F1T1p2
	F2T0p2 = (pint/3)*F2T0p2
	F2T1p2 = (pint/3)*F2T1p2
	F0T0E = (pint/3)*F0T0E
	F0T1E = (pint/3)*F0T1E
	F1T0E = (pint/3)*F1T0E
	F1T1E = (pint/3)*F1T1E

c	N0n=0.980526	!0.990473
c	N0p=1.98297	!1.99339
c	norm0 = 3./(2.*N0p-N0n)
c	norm1 = 1./N0n
c	F0T0 = norm0 * F0T0
c	F0T1 = norm1 * F0T1
c
c	print *, 'F2T0,F2T1=',4*pi*f2t0,4*pi*f2t1

	N0p = (4*pi) * (F0T0 + F0T1)
	N0n = (4*pi) * 2*F0T1
	N1p = (4*pi) * (F1T0 + F1T1)
	N1n = (4*pi) * 2*F1T1
	N2p = (4*pi) * (F2T0 + F2T1)
	N2n = (4*pi) * 2*F2T1
	T0p = (4*pi) * (F0T0p2 + F0T1p2)
	T0n = (4*pi) * 2*F0T1p2
	T1p = (4*pi) * (F1T0p2 + F1T1p2)
	T1n = (4*pi) * 2*F1T1p2
	T2p = (4*pi) * (F2T0p2 + F2T1p2)
	T2n = (4*pi) * 2*F2T1p2
	E0p = -(4*pi) * (F0T0E + F0T1E)  -  T0p/4
	E0n = -(4*pi) * 2*F0T1E  -  T0n/4
	E1p = -(4*pi) * (F1T0E + F1T1E)  -  T1p/4
	E1n = -(4*pi) * 2*F1T1E  -  T1n/4

	print *, '--------------------------------------'
	print *, 'N0p,N0n=',n0p,n0n
	print *, 'norm0,norm1=',norm0,norm1
	print *
	print *, 'N1p,N1n=',n1p,n1n
	print *
	print *, 'N2p,N2n=',n2p,n2n
	print *
	print *, 'T0p,T0n=',t0p,t0n
	print *, 'T1p,T1n=',t1p,t1n
	print *, 'T2p,T2n=',t2p,t2n
	print *
	print *, 'E0p,E0n(MeV)=',e0p*mN*hc,e0n*mN*hc
	print *, 'E1p,E1n=',e1p,e1n
	print *
	print *, '--------------------------------------'
	print *


C........................................................................
C...Calculate structure function.........................................
	Gam1n = 0.0
	Gam1p = 0.0
	Gam1A_n = 0.0
	Gam1A_p = 0.0
	Gam2n = 0.0
	Gam2p = 0.0
	Gam2A_n = 0.0
	Gam2A_p = 0.0
	d2n = 0.0
	d2p = 0.0
	d2A_n = 0.0
	d2A_p = 0.0

	xmin = xmax/nx
	xmax = 1.0
	xint = xmax/nx
	ix = 1
	DO x=xint,xmax-xint/2,xint

	  xN(ix) = x

	  gam = SQRT( 1.0 + 4*mN2*x**2/Q2 )
	  gam2 = 1.0 + 2*mN2*x/Q2

	  qvec = Q2/(2*mN*x) * gam		! 1/fm
	  ss = M3**2 + Q2*(M3/mN/x - 1.0)	! 1/fm^2
	  b = qvec/SQRT(ss+qvec**2)

	  W = SQRT(mN2 + Q2*(1.0-x)/x)*hc	! MeV
c	print *, 'Q2(GeV2),x,W(GeV)=',q2*hc2/1.E6,x,w/1.E3

C...Nucleon structure function
	  CALL MAID_G1G2 (W,Q2*hc2,G1,G2,1)	! 1=p, 2=n
	  xg1p(ix) = x*G1
	  xg2p(ix) = x*G2
	  CALL MAID_G1G2 (W,Q2*hc2,G1,G2,2)
	  xg1n(ix) = x*G1
	  xg2n(ix) = x*G2

c	  CALL LSS_S (DBLE(x),dxg1p,dxg1n)
c	  CALL G1PAR (dble(x),dble(Q2gev),dxg1p,dxg1n)
c	  CALL G2PAR (dble(x),dble(Q2gev),dxg2p,dxg2n)
c	  xg1p(ix) = SNGL(dxg1p)
c	  xg1n(ix) = SNGL(dxg1n)
c
c	  Au = 3.088
c	  Ad = 0.343
c	  Bu = -3.010
c	  Bd = -0.265
c	  Cu = 2.143
c	  Cd = 1.689
c	  Du = -2.065
c	  Dd = -1.610
c	  xDu = x**(-0.313) * (1.0-x)**3.0
c     &	      * ( Au + Bu*(1.0-x) - Cu*(1.0-x)**2 - Du*(1.0-x)**3 )
c	  xDd = x**(-0.313) * (1.0-x)**3.0
c     &        * ( Ad + Bd*(1.0-x) - Cd*(1.0-x)**2 - Dd*(1.0-x)**3 )
c	  xg1p(ix) = (4.0*xDu + xDd)/18.0
c	  xg1n(ix) = (xDu + 4.0*xDd)/18.0
c
c	  xg1p(ix) = eu * 0.954 / 0.44746 * x**0.8 * (1.0-x)**2.4
c     &		   - ed * 0.278 / 0.36223 * x**0.8 * (1.0-x)**3.4
c	  xg1n(ix) = ed * 0.954 / 0.44746 * x**0.8 * (1.0-x)**2.4
c     &		   - eu * 0.278 / 0.36223 * x**0.8 * (1.0-x)**3.4
c	  xg2p(ix) = -xg1p(ix)
c     &		   + eu * 0.954 / 0.44746 * (-7.15936 + 6.0/x**0.2)
c     &		   - ed * 0.278 / 0.36223 * (-7.60682 + 6.4/x**0.2)
c	  xg2n(ix) = -xg1n(ix)
c     &		   + ed * 0.954 / 0.44746 * (-7.15936 + 6.0/x**0.2)
c     &		   - eu * 0.278 / 0.36223 * (-7.60682 + 6.4/x**0.2)


C...g2(WW) model
c	  xg2p(ix) = 0.0
c	  xg2n(ix) = 0.0
c	  ny = 50
c	  yint = (1.0-x)/ny
c	  iy = 0
c	  DO y=x,1.0-2*yint,yint
c
c	    CALL LSS_S (DBLE(y),dxg1p,dxg1n)
c	    xDu = y**(-0.313) * (1.0-y)**3.0
c     &		* ( Au + Bu*(1.0-y) - Cu*(1.0-y)**2 - Du*(1.0-y)**3 )
c	    xDd = y**(-0.313) * (1.0-y)**3.0
c     &		* ( Ad + Bd*(1.0-y) - Cd*(1.0-y)**2 - Dd*(1.0-y)**3 )
c
c	    IF (iy.EQ.0 .OR. iy.EQ.ny) THEN
c	      xg2p(ix) = xg2p(ix) + x*SNGL(dxg1p)/y**2
c	      xg2n(ix) = xg2n(ix) + x*SNGL(dxg1n)/y**2
c	      xg2p(ix) = xg2p(ix) + x*(4.0*xDu + xDd)/18.0/y**2
c	      xg2n(ix) = xg2n(ix) + x*(xDu + 4.0*xDd)/18.0/y**2
c	    ELSE IF (iy/2*2.NE.iy) THEN
c	      xg2p(ix) = xg2p(ix) + 2*x*SNGL(dxg1p)/y**2
c	      xg2n(ix) = xg2n(ix) + 2*x*SNGL(dxg1n)/y**2
c	      xg2p(ix) = xg2p(ix) + 2*x*(4.0*xDu + xDd)/18.0/y**2
c	      xg2n(ix) = xg2n(ix) + 2*x*(xDu + 4.0*xDd)/18.0/y**2
c	    ELSE IF (iy/2*2.EQ.iy) THEN
c	      xg2p(ix) = xg2p(ix) + 4*x*SNGL(dxg1p)/y**2
c	      xg2n(ix) = xg2n(ix) + 4*x*SNGL(dxg1n)/y**2
c	      xg2p(ix) = xg2p(ix) + 4*x*(4.0*xDu + xDd)/18.0/y**2
c	      xg2n(ix) = xg2n(ix) + 4*x*(xDu + 4.0*xDd)/18.0/y**2
c	    ENDIF
c	    iy = iy + 1
c
c	  ENDDO
c	  xg2p(ix) = (yint/3)*xg2p(ix) - xg1p(ix)
c	  xg2n(ix) = (yint/3)*xg2n(ix) - xg1n(ix)

c	print *, '*** xg1n,xg1p=',xg1n(ix),xg1p(ix)
c	print *, '*** xg2n,xg2p=',xg2n(ix),xg2p(ix)


c	if(.true.)goto 999

C...Three-body contribution..............................................
C...Evaluate energy (MeV) array
C...NB: e = E_{sauer} = -epsilon - p^2/4mN , where p0 = M + epsilon

	  DO IEN=2,NERG

	    e = E_ARRAY(IEN-1)

	    DO JQ=1,NQ				! momentum array
	      P0T0ar(JQ) = R0T0(IEN,JQ)		! for fixed energy
	      P0T1ar(JQ) = R0T1(IEN,JQ)
	      P1T0ar(JQ) = R1T0(IEN,JQ)
	      P1T1ar(JQ) = R1T1(IEN,JQ)
	      P2T0ar(JQ) = R2T0(IEN,JQ)
	      P2T1ar(JQ) = R2T1(IEN,JQ)
	    ENDDO

	    M2 = 2*mN + E_B/hc + e/hc	! 1/fm^2
	    a = ( ss + M2**2 - mN2 ) / ( 2*mN*SQRT(ss+qvec**2) )

C...Momentum integration (all momenta in 1/fm)
	    pTmin = 0.0
	    pTmax = MIN((a**2 + M2**2/mN2*(b**2-1.0))/(1.0-b**2)*mN, 4.0)

	    npT = 100
	    pTint = (pTmax-pTmin)/npT
	    pp0_T = 0.0
	    nn0_T = 0.0
	    pp1_T = 0.0
	    nn1_T = 0.0
	    pp2_T = 0.0
	    nn2_T = 0.0
	    ipT = 1
	    DO pT=pTint,pTmax,pTint

	      pT2 = pT**2
	      px2 = pT2/2	! integration over d(phi) implicit

	      pzmin = ( a*b - SQRT(a**2-(1.0-b**2)*(M2**2/mN2+pT2/mN2)) )
     &		    / (1.0-b**2) * mN
	      pzmax = MIN( ( a*b + SQRT(a**2-(1.0-b**2)*(M2**2/mN2+pT2/mN2)) )
     &			   / (1.0-b**2) * mN, 4.0 )

	      npz = 100
	      pzint = (pzmax-pzmin)/npz
	      pp0_z = 0.0
	      nn0_z = 0.0
	      pp1_z = 0.0
	      nn1_z = 0.0
	      pp2_z = 0.0
	      nn2_z = 0.0
	      ipz = 0
	      DO pz=pzmin,pzmax,pzint

		pz2 = pz**2
		p = SQRT(pT2 + pz2)
		p2 = p**2

		f0T0 = QDVAL(p,NQ,P_ARRAY,P0T0ar,.FALSE.)
		f0T1 = QDVAL(p,NQ,P_ARRAY,P0T1ar,.FALSE.)
		f1T0 = QDVAL(p,NQ,P_ARRAY,P1T0ar,.FALSE.)
		f1T1 = QDVAL(p,NQ,P_ARRAY,P1T1ar,.FALSE.)
		f2T0 = QDVAL(p,NQ,P_ARRAY,P2T0ar,.FALSE.)
		f2T1 = QDVAL(p,NQ,P_ARRAY,P2T1ar,.FALSE.)

		f0p = f0T0 + f0T1
		f0n = 2*f0T1
		f1p = f1T0 + f1T1
		f1n = 2*f1T1
		f2p = f2T0 + f2T1
		f2n = 2*f2T1

		eps = -e/hc - p2/(4.0*mN)		! eps in 1/fm
		z = x / (1.0 + (eps + gam*pz)/mN)
		W = SQRT(mN2 + Q2*(1.0-z)/z)*hc		! MeV

		IF (z.GE.1.0 .OR. z.LT.x/3.0) GOTO 123

C...Bound nucleon structure function
		CALL MAID_G1G2 (W,Q2*hc2,G1,G2,1)	! 1=p, 2=n
		zg1p = z*G1
		zg2p = z*G2
		CALL MAID_G1G2 (W,Q2*hc2,G1,G2,2)
		zg1n = z*G1
		zg2n = z*G2

c		CALL LSS_S (DBLE(z),dxg1p,dxg1n)
c		CALL G1PAR (dble(z),dble(Q2gev),dxg1p,dxg1n)
c		CALL G2PAR (dble(z),dble(Q2gev),dxg1p,dxg2n)
c		zg1p = SNGL(dxg1p)
c		zg1n = SNGL(dxg1n)
c
c		xDu = z**(-0.313) * (1.0-z)**3.0
c     &		    * ( Au + Bu*(1.0-z) - Cu*(1.0-z)**2 - Du*(1.0-z)**3 )
c		xDd = z**(-0.313) * (1.0-z)**3.0
c     &		    * ( Ad + Bd*(1.0-z) - Cd*(1.0-z)**2 - Dd*(1.0-z)**3 )
c		zg1p = (4.0*xDu + xDd)/18.0
c		zg1n = (xDu + 4.0*xDd)/18.0
c
c		zg1p = eu * 0.954 / 0.44746 * z**0.8 * (1.0-z)**2.4
c     &		     - ed * 0.278 / 0.36223 * z**0.8 * (1.0-z)**3.4
c		zg1n = ed * 0.954 / 0.44746 * z**0.8 * (1.0-z)**2.4
c     &		     - eu * 0.278 / 0.36223 * z**0.8 * (1.0-z)**3.4
c		zg2p = -zg1p
c     &		     + eu * 0.954 / 0.44746 * (-7.15936 + 6.0/z**0.2)
c     &		     - ed * 0.278 / 0.36223 * (-7.60682 + 6.4/z**0.2)
c		zg2n = -zg1n
c     &		     + ed * 0.954 / 0.44746 * (-7.15936 + 6.0/z**0.2)
c     &		     - eu * 0.278 / 0.36223 * (-7.60682 + 6.4/z**0.2)


C...g2(WW) model
c		zg2p = 0.0
c		zg2n = 0.0
c		ny = 50
c		yint = (1.0-z)/ny
c		iy = 0
c		DO y=z,1.0-2*yint,yint
c
c		  CALL LSS_S (DBLE(y),dxg1p,dxg1n)
c		  xDu = y**(-0.313) * (1.0-y)**3.0
c     &		      * ( Au + Bu*(1.0-y) - Cu*(1.0-y)**2 - Du*(1.0-y)**3 )
c		  xDd = y**(-0.313) * (1.0-y)**3.0
c     &		      * ( Ad + Bd*(1.0-y) - Cd*(1.0-y)**2 - Dd*(1.0-y)**3 )
c
c		  IF (iy.EQ.0 .OR. iy.EQ.ny) THEN
c		    zg2p = zg2p + z*(4.0*xDu + xDd)/18.0/y**2
c		    zg2n = zg2n + z*(xDu + 4.0*xDd)/18.0/y**2
c		  ELSE IF (iy/2*2.NE.iy) THEN
c		    zg2p = zg2p + 2*z*(4.0*xDu + xDd)/18.0/y**2
c		    zg2n = zg2n + 2*z*(xDu + 4.0*xDd)/18.0/y**2
c		  ELSE IF (iy/2*2.EQ.iy) THEN
c		    zg2p = zg2p + 4*z*(4.0*xDu + xDd)/18.0/y**2
c		    zg2n = zg2n + 4*z*(xDu + 4.0*xDd)/18.0/y**2
c		  ENDIF
c		  iy = iy + 1
c
c		ENDDO
c
c		zg2p = (yint/3)*zg2p - zg1p
c		zg2n = (yint/3)*zg2n - zg1n

		prl_p = (1.0-eps/mN)
     &		      * ( ( f1p * (1.0 + gam*pz/mN + pz2/mN2)
     &			  + f2p * (pz2/p2 - 1.0/3.0
     &				  + 2*gam*pz/(3*mN)
     &				  + 2*pz2/(3*mN2)) )
     &			  * zg1p
     &			+ (1.0-gam**2)*(1+eps/mN)
     &			  * ( f1p + f2p * (pz2/p2-1.0/3.0) )
     &			  * (z/x)*zg2p )
     &		      * (2*pi*pT)
		prl_n = (1.0-eps/mN)
     &		      * ( ( f1n * (1.0 + gam*pz/mN + pz2/mN2)
     &			  + f2n * (pz2/p2 - 1.0/3.0
     &				  + 2*gam*pz/(3*mN)
     &				  + 2*pz2/(3*mN2)) )
     &			  * zg1n *0.0!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     &			+ (1.0-gam**2)*(1+eps/mN)
     &			  * ( f1n + f2n * (pz2/p2-1.0/3.0) )
     &			  * (z/x)*zg2n )
     &		      * (2*pi*pT)

		prp_p = (1.0-eps/mN)
     &		      * ( ( f1p * (1.0 + px2/mN2)
     &			  + f2p * (px2/p2 - 1.0/3.0 + 2*px2/(3*mN2)) )
     &			  * zg1p
     &			+ ( f1p * (1.0 + px2/mN2*(1.0-z/x))
     &			  + f2p * (px2/p2 - 1.0/3.0
     &				   + 2*px2/(3*mN2)*(1-z/x)
     &				   - gam*pz/mN*(px2/p2)*z/x) )
     &			  * (z/x)*zg2p )
     &		      * (2*pi*pT)
		prp_n = (1.0-eps/mN)
     &		      * ( ( f1n * (1.0 + px2/mN2)
     &			  + f2n * (px2/p2 - 1.0/3.0 + 2*px2/(3*mN2)) )
     &			  * zg1n *0.0!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     &			+ ( f1n * (1.0 + px2/mN2*(1.0-z/x))
     &			  + f2n * (px2/p2 - 1.0/3.0
     &				   + 2*px2/(3*mN2)*(1-z/x)
     &				   - gam*pz/mN*(px2/p2)*z/x) )
     &			  * (z/x)*zg2n )
     &		      * (2*pi*pT)

		int1p = (prl_p + (gam**2-1.0)*prp_p)/gam**2
		int1n = (prl_n + (gam**2-1.0)*prp_n)/gam**2
		int2p = (prp_p - prl_p)/gam**2
		int2n = (prp_n - prl_n)/gam**2

		IF (ipz.EQ.0 .OR. ipz.EQ.npz) THEN
		  pp0_z = pp0_z + (2*pi*pT)*f1p
		  nn0_z = nn0_z + (2*pi*pT)*f1n
		  pp1_z = pp1_z + int1p
		  nn1_z = nn1_z + int1n
		  pp2_z = pp2_z + int2p
		  nn2_z = nn2_z + int2n
		ELSE IF (ipz/2*2.NE.ipz) THEN
		  pp0_z = pp0_z + 4*(2*pi*pT)*f1p
		  nn0_z = nn0_z + 4*(2*pi*pT)*f1n
		  pp1_z = pp1_z + 4*int1p
		  nn1_z = nn1_z + 4*int1n
		  pp2_z = pp2_z + 4*int2p
		  nn2_z = nn2_z + 4*int2n
		ELSE IF (ipz/2*2.EQ.ipz) THEN
		  pp0_z = pp0_z + 2*(2*pi*pT)*f1p
		  nn0_z = nn0_z + 2*(2*pi*pT)*f1n
		  pp1_z = pp1_z + 2*int1p
		  nn1_z = nn1_z + 2*int1n
		  pp2_z = pp2_z + 2*int2p
		  nn2_z = nn2_z + 2*int2n
		ENDIF

 123	        ipz = ipz + 1

	      ENDDO				! pz loop
	      pp0_z = (pzint/3)*pp0_z		! (-inf,inf) -> 2*(0,inf)
	      nn0_z = (pzint/3)*nn0_z
	      pp1_z = (pzint/3)*pp1_z
	      nn1_z = (pzint/3)*nn1_z
	      pp2_z = (pzint/3)*pp2_z
	      nn2_z = (pzint/3)*nn2_z

	      IF (ipT/2*2.NE.ipT) THEN
		pp0_T = pp0_T + 4*pp0_z
		nn0_T = nn0_T + 4*nn0_z
		pp1_T = pp1_T + 4*pp1_z
		nn1_T = nn1_T + 4*nn1_z
		pp2_T = pp2_T + 4*pp2_z
		nn2_T = nn2_T + 4*nn2_z
	      ELSE IF (ipT/2*2.EQ.ipT) THEN
		pp0_T = pp0_T + 2*pp0_z
		nn0_T = nn0_T + 2*nn0_z
		pp1_T = pp1_T + 2*pp1_z
		nn1_T = nn1_T + 2*nn1_z
		pp2_T = pp2_T + 2*pp2_z
		nn2_T = nn2_T + 2*nn2_z
	      ENDIF

	      ipT = ipT + 1

	    ENDDO				! pT loop
	    Ep0ar(IEN-1) = (pTint/3)*pp0_T
	    En0ar(IEN-1) = (pTint/3)*nn0_T
	    Ep1ar(IEN-1) = (pTint/3)*pp1_T
	    En1ar(IEN-1) = (pTint/3)*nn1_T
	    Ep2ar(IEN-1) = (pTint/3)*pp2_T
	    En2ar(IEN-1) = (pTint/3)*nn2_T

	  ENDDO					! e array loop

C...Integrate over energy
	  emin = 0.01
	  emax = MIN(SQRT(ss)*hc - 2*mN*hc - E_B - mN*hc, 68.0)
	  ne = 1000
	  eint = (emax-emin)/ne
	  pp0 = 0.0
	  nn0 = 0.0
	  pp1 = 0.0
	  nn1 = 0.0
	  pp2 = 0.0
	  nn2 = 0.0
	  ie = 0
	  DO e=emin,emax,eint

	    p0 = QDVAL(e,NERG-1,E_ARRAY,Ep0ar,.FALSE.)
	    n0 = QDVAL(e,NERG-1,E_ARRAY,En0ar,.FALSE.)
	    p1 = QDVAL(e,NERG-1,E_ARRAY,Ep1ar,.FALSE.)
	    n1 = QDVAL(e,NERG-1,E_ARRAY,En1ar,.FALSE.)
	    p2 = QDVAL(e,NERG-1,E_ARRAY,Ep2ar,.FALSE.)
	    n2 = QDVAL(e,NERG-1,E_ARRAY,En2ar,.FALSE.)

	    IF (n0.LE.0.0 .OR. p0.LE.0.0) GOTO 495

	    IF (ie.EQ.0 .OR. ie.EQ.ne) THEN
	      pp0 = pp0 + p0
	      nn0 = nn0 + n0
	      pp1 = pp1 + p1
	      nn1 = nn1 + n1
	      pp2 = pp2 + p2
	      nn2 = nn2 + n2
	    ELSE IF (ie/2*2.NE.ie) THEN
	      pp0 = pp0 + 4*p0
	      nn0 = nn0 + 4*n0
	      pp1 = pp1 + 4*p1
	      nn1 = nn1 + 4*n1
	      pp2 = pp2 + 4*p2
	      nn2 = nn2 + 4*n2
	    ELSE IF (ie/2*2.EQ.ie) THEN
	      pp0 = pp0 + 2*p0
	      nn0 = nn0 + 2*n0
	      pp1 = pp1 + 2*p1
	      nn1 = nn1 + 2*n1
	      pp2 = pp2 + 2*p2
	      nn2 = nn2 + 2*n2
	    ENDIF

	    ie = ie + 1

	  ENDDO					! e loop
 495	  pp0 = (eint/3)*pp0
	  nn0 = (eint/3)*nn0
	  xg1A_p(ix) = (eint/3)*pp1
	  xg1A_n(ix) = (eint/3)*nn1
	  xg2A_p(ix) = (eint/3)*pp2
	  xg2A_n(ix) = (eint/3)*nn2

	print *, '>> x,pol: n=',x,nn0
	print *, '==(total)== g1: n, n(He) =',
     &		xg1n(ix),xg1A_n(ix)
	print *, '==(total)== g2: n, n(He) =',
     &		xg2n(ix),xg2A_n(ix)
	print *


	  if(.true.)goto 999
C...Deuteron channel contribution........................................
C...Energy integration fixed, e = -E_B + E_d
	  M2 = 2*mN + E_d/hc
	  a = ( ss + M2**2 - mN2 ) / ( 2*mN*SQRT(ss+qvec**2) )

C...Momentum integration (all momenta in 1/fm)
	  pTmin = 0.0
	  pTmax = MIN((a**2 + M2**2/mN2*(b**2-1.0))/(1.0-b**2)*mN, 4.0)

	  npT = 100
	  pTint = (pTmax-pTmin)/npT
	  pp0_T = 0.0
	  pp1_T = 0.0
	  pp2_T = 0.0
	  ipT = 1
	  DO pT=pTint,pTmax,pTint

	    pT2 = pT**2
	    px2 = pT2/2		! integration over d(phi) implicit

	    pzmin = ( a*b - SQRT(a**2-(1.0-b**2)*(M2**2/mN2+pT2/mN2)) )
     &		  / (1.0-b**2) * mN
	    pzmax = MIN( ( a*b + SQRT(a**2-(1.0-b**2)*(M2**2/mN2+pT2/mN2)) )
     &			 / (1.0-b**2) * mN, 4.0 )

	    npz = 100
	    pzint = (pzmax-pzmin)/npz
	    pp0_z = 0.0
	    pp1_z = 0.0
	    pp2_z = 0.0
	    ipz = 0
	    DO pz=pzmin,pzmax,pzint

	      pz2 = pz**2
	      p = SQRT(pT2 + pz2)
	      p2 = p**2

	      eps2 = -E_B/hc + E_d/hc			! 1/fm
	      z2 = x / (1.0 + (eps2 + gam*pz)/mN)

c	      CALL LSS_S (DBLE(z2),dxg1p,dxg1n)
	      xDu = z2**(-0.313) * (1.0-z2)**3.0
     &		  * ( Au + Bu*(1.0-z2) - Cu*(1.0-z2)**2 - Du*(1.0-z2)**3 )
	      xDd = z2**(-0.313) * (1.0-z2)**3.0
     &		  * ( Ad + Bd*(1.0-z2) - Cd*(1.0-z2)**2 - Dd*(1.0-z2)**3 )
	      zg1p = (4*xDu + xDd)/18

c	      zg1p = eu * 0.954 / 0.44746 * z2**0.8 * (1.0-z2)**2.4
c     &		   - ed * 0.278 / 0.36223 * z2**0.8 * (1.0-z2)**3.4
c	      zg2p = -zg1p
c     &		   + eu * 0.954 / 0.44746 * (-7.15936 + 6.0/z2**0.2)
c     &		   - ed * 0.278 / 0.36223 * (-7.60682 + 6.4/z2**0.2)


C...g2(WW) model
c	      zg2p = 0.0
c	      ny = 20
c	      yint = (1.0-z2)/ny
c	      iy = 0
c	      DO y=z2,1.0-2*yint,yint
c
c		CALL LSS_S (DBLE(y),dxg1p,dxg1n)
c		xDu = y**(-0.313) * (1.0-y)**3.0
c     &		    * ( Au + Bu*(1.0-y) - Cu*(1.0-y)**2 - Du*(1.0-y)**3 )
c		xDd = y**(-0.313) * (1.0-y)**3.0
c     &		    * ( Ad + Bd*(1.0-y) - Cd*(1.0-y)**2 - Dd*(1.0-y)**3 )
c
c		IF (iy.EQ.0 .OR. iy.EQ.ny) THEN
c		  zg2p = zg2p + z2*(4*xDu + xDd)/18/y**2
c		ELSE IF (iy/2*2.NE.iy) THEN
c		  zg2p = zg2p + 2*z2*(4*xDu + xDd)/18/y**2
c		ELSE IF (iy/2*2.EQ.iy) THEN
c		  zg2p = zg2p + 4*z2*(4*xDu + xDd)/18/y**2
c		ENDIF
c
c	      ENDDO
c	      zg2p = (yint/3)*zg2p - zg1p

	      f0T0 = QDVAL(p,NQ,P_ARRAY,TWO0T0,.FALSE.)
	      f1T0 = QDVAL(p,NQ,P_ARRAY,TWO1T0,.FALSE.)
	      f2T0 = QDVAL(p,NQ,P_ARRAY,TWO2T0,.FALSE.)

	      prl_p = (1.0-eps/mN)
     &		    * ( ( f1T0 * (1.0 + gam*pz/mN + pz2/mN2)
     &			+ f2T0*0 * (pz2/p2 - 1.0/3.0
     &				 + 2*gam*pz/(3*mN)
     &				 + 2*pz2/(3*mN2)) )
     &			* zg1p
     &		      + (1-gam**2)*(1+eps/mN)
     &			* ( f1T0 + f2T0 * (pz2/p2-1.0/3.0) )
     &			* (z/x)*zg2p )
     &		    * (2*pi*pT)
	      prp_p = (1.0-eps/mN)
     &		    * ( ( f1T0 * (1.0 + px2/mN2)
     &			+ f2T0*0 * (px2/p2 - 1.0/3.0 + 2*px2/(3*mN2)) )
     &			* zg1p
     &		      + ( f1T0 * (1.0 + px2/mN2*(1.0-z/x))
     &			+ f2T0 * (px2/p2 - 1.0/3.0
     &				 + 2*px2/(3*mN2)*(1-z/x)
     &				 - gam*pz/mN*(px2/p2)*z/x) )
     &			* (z/x)*zg2p )
     &		    * (2*pi*pT)

	      int1p = (prl_p + (gam**2-1.0)*prp_p)/gam**2
	      int2p = (prp_p - prl_p)/gam**2

	      IF (ipz.EQ.0 .OR. ipz.EQ.npz) THEN
		pp0_z = pp0_z + (2*pi*pT)*f1T0
		pp1_z = pp1_z + int1p
		pp2_z = pp2_z + int2p
	      ELSE IF (ipz/2*2.NE.ipz) THEN
		pp0_z = pp0_z + 4*(2*pi*pT)*f1T0
		pp1_z = pp1_z + 4*int1p
		pp2_z = pp2_z + 4*int2p
	      ELSE IF (ipz/2*2.EQ.ipz) THEN
		pp0_z = pp0_z + 2*(2*pi*pT)*f1T0
		pp1_z = pp1_z + 2*int1p
		pp2_z = pp2_z + 2*int2p
	      ENDIF

	      ipz = ipz + 1

	    ENDDO				! pz loop
	    pp0_z = (pzint/3)*pp0_z
	    pp1_z = (pzint/3)*pp1_z
	    pp2_z = (pzint/3)*pp2_z

	    IF (ipT/2*2.NE.ipT) THEN
	      pp0_T = pp0_T + 4*pp0_z
	      pp1_T = pp1_T + 4*pp1_z
	      pp2_T = pp2_T + 4*pp2_z
	    ELSE IF (ipT/2*2.EQ.ipT) THEN
	      pp0_T = pp0_T + 2*pp0_z
	      pp1_T = pp1_T + 2*pp1_z
	      pp2_T = pp2_T + 2*pp2_z
	    ENDIF

	    ipT = ipT + 1

	  ENDDO					! pT loop
C...additional (2-body) contribution to proton structure function
	  pp0 = (pTint/3)*pp0_T + pp0
	  xg1A_p(ix) = (pTint/3)*pp1_T + xg1A_p(ix)
	  xg2A_p(ix) = (pTint/3)*pp2_T + xg2A_p(ix)


c	print *, '>> pol: n, p=',nn0,pp0
c	print *, '==(total)== g1: n, n(He), p(He) =',
c     &		xg1n(ix),xg1A_n(ix),xg1A_p(ix)
c	print *, '==(total)== g2: n, n(He), p(He) =',
c     &		xg2n(ix),xg2A_n(ix),xg2A_p(ix)
c	print *

 999	continue

	  IF (ix/2*2.NE.ix) THEN
	    Gam1n   = Gam1n   + 4*xg1n(ix)/xN(ix)
	    Gam1p   = Gam1p   + 4*xg1p(ix)/xN(ix)
	    Gam1A_n = Gam1A_n + 4*xg1A_n(ix)/xN(ix)
	    Gam1A_p = Gam1A_p + 4*xg1A_p(ix)/xN(ix)
	    Gam2n   = Gam2n   + 4*xg2n(ix)/xN(ix)
	    Gam2p   = Gam2p   + 4*xg2p(ix)/xN(ix)
	    Gam2A_n = Gam2A_n + 4*xg2A_n(ix)/xN(ix)
	    Gam2A_p = Gam2A_p + 4*xg2A_p(ix)/xN(ix)
	    d2n     = d2n     + 4*(2*xg1n(ix)  +3*xg2n(ix))  *xN(ix)
	    d2p     = d2p     + 4*(2*xg1p(ix)  +3*xg2p(ix))  *xN(ix)
	    d2A_n   = d2A_n   + 4*(2*xg1A_n(ix)+3*xg2A_n(ix))*xN(ix)
	    d2A_p   = d2A_p   + 4*(2*xg1A_p(ix)+3*xg2A_p(ix))*xN(ix)
	  ELSE IF (ix/2*2.EQ.ix) THEN
	    Gam1n   = Gam1n   + 2*xg1n(ix)/xN(ix)
	    Gam1p   = Gam1p   + 2*xg1p(ix)/xN(ix)
	    Gam1A_n = Gam1A_n + 2*xg1A_n(ix)/xN(ix)
	    Gam1A_p = Gam1A_p + 2*xg1A_p(ix)/xN(ix)
	    Gam2n   = Gam2n   + 2*xg2n(ix)/xN(ix)
	    Gam2p   = Gam2p   + 2*xg2p(ix)/xN(ix)
	    Gam2A_n = Gam2A_n + 2*xg2A_n(ix)/xN(ix)
	    Gam2A_p = Gam2A_p + 2*xg2A_p(ix)/xN(ix)
	    d2n     = d2n     + 2*(2*xg1n(ix)  +3*xg2n(ix))  *xN(ix)
	    d2p     = d2p     + 2*(2*xg1p(ix)  +3*xg2p(ix))  *xN(ix)
	    d2A_n   = d2A_n   + 2*(2*xg1A_n(ix)+3*xg2A_n(ix))*xN(ix)
	    d2A_p   = d2A_p   + 2*(2*xg1A_p(ix)+3*xg2A_p(ix))*xN(ix)
	  ENDIF

	  ix = ix + 1

	ENDDO			! x loop
	Gam1n   = (xint/3)*Gam1n
	Gam1p   = (xint/3)*Gam1p
	Gam1A_n = (xint/3)*Gam1A_n
	Gam1A_p = (xint/3)*Gam1A_p
	Gam2n   = (xint/3)*Gam2n
	Gam2p   = (xint/3)*Gam2p
	Gam2A_n = (xint/3)*Gam2A_n
	Gam2A_p = (xint/3)*Gam2A_p
	d2n     = (xint/3)*d2n
	d2p     = (xint/3)*d2p
	d2A_n   = (xint/3)*d2A_n
	d2A_p   = (xint/3)*d2A_p

	print *, 'Q2(GeV2)=',q2gev
	print *, 'Gam1 (n,A_n(NOG1))=',Gam1n,Gam1A_n
	print *, 'Gam2 (n,A_n(NOG1))=',Gam2n,Gam2A_n
	print *, 'd2 (n,A_n(NOG1))=',d2n,d2A_n
c	close(88)

	enddo

c	if(.true.)stop
C...Write to file
c	OPEN (11,FILE='xg1n.Q026',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (12,FILE='xg1p.',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (13,FILE='xg1A_n.Q026',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (14,FILE='xg1A_p.',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (15,FILE='xg1A.',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (21,FILE='xg2n.Q026',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (22,FILE='xg2p.',STATUS='UNKNOWN',FORM='FORMATTED')
	OPEN (23,FILE='xg2A_n.Q2',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (24,FILE='xg2A_p.',STATUS='UNKNOWN',FORM='FORMATTED')
c	OPEN (25,FILE='xg2A.',STATUS='UNKNOWN',FORM='FORMATTED')
	DO ix=0,nx
c	  WRITE (11,*) xN(ix), xg1n(ix)
c	  WRITE (12,*) xN(ix), xg1p(ix)
c	  WRITE (13,*) xN(ix), xg1A_n(ix)
c	  WRITE (14,*) xN(ix), xg1A_p(ix)
c	  WRITE (15,*) xN(ix), xg1A_n(ix)+xg1A_p(ix)
c	  WRITE (21,*) xN(ix), xg2n(ix)
c	  WRITE (22,*) xN(ix), xg2p(ix)
	  WRITE (23,*) xN(ix), xg2A_n(ix)
c	  WRITE (24,*) xN(ix), xg2A_p(ix)
c	  WRITE (25,*) xN(ix), xg2A_n(ix)+xg2A_p(ix)
	ENDDO
c	CLOSE (11)
c	CLOSE (12)
c	CLOSE (13)
c	CLOSE (14)
c	CLOSE (15)
c	CLOSE (21)
c	CLOSE (22)
	CLOSE (23)
c	CLOSE (24)
c	CLOSE (25)

	STOP
	END


C **********************************************************************
	SUBROUTINE G1PAR (x,Q2,xg1p,xg1n)
C
C  g1 structure function and derivatives
C **********************************************************************
	IMPLICIT NONE
        REAL*8  x,Q2,xg1p,xg1n,Dxg1N,DDxg1N,Dp2xg1N
        REAL*8  h,XG1NPAR
        REAL*8  s,M,xx,xxmax,xxmin,xxint,g1N,dphi_dp2,norm,nume
        INTEGER nxx,ixx
        LOGICAL p2norm /.FALSE./
        COMMON  /phi/ dphi_dp2

	if(.true.)goto 184

        IF (p2norm) GO TO 184
C...Derivative of \phi(p2) wrt p2
        M = 938.91897D0	  ! Nucleon mass (MeV)
        s = 40.0D0 * 1.D6 ! Mass squared of spectator system (MeV^2)
        xxmax = 1.D0
        xxmin = 0.D0
        nxx = 100000
        xxint = (xxmax-xxmin)/DFLOAT(nxx)

        dphi_dp2 = 0.D0
        norm = 0.D0
        ixx = 1
        DO xx=xxint,xxmax,xxint
          g1N = XG1NPAR(xx,Q2)/xx
          nume = g1N * (1.D0-xx)
     &         * ( (3.D0*xx - 1.D0)*s + M**2*(1.D0-xx)**3 )
     &         / (M**2*(1.D0-xx)**2 - s)**2
          IF (ixx.EQ.0 .OR. ixx.EQ.nxx) THEN
            norm     = norm     + g1N
            dphi_dp2 = dphi_dp2 + nume
          ELSE IF (ixx/2*2.NE.ixx) THEN
            norm     = norm     + 4.D0*g1N
            dphi_dp2 = dphi_dp2 + 4.D0*nume
          ELSE IF (ixx/2*2.EQ.ixx) THEN
            norm     = norm     + 2.D0*g1N
            dphi_dp2 = dphi_dp2 + 2.D0*nume
          ENDIF
          ixx = ixx + 1
        ENDDO
        norm     = (xxint/3.D0) * norm
        dphi_dp2 = (xxint/3.D0) * dphi_dp2 / norm
        print *, 'norm(g1N),s,dphi_dp2(GeV-2)=',norm,s,dphi_dp2*1.D6
        p2norm = .TRUE.


 184    h = 0.001D0
C...Value of x*g1N
        xg1p = XG1NPAR(1,x,Q2)
        xg1n = XG1NPAR(0,x,Q2)
C...First derivative wrt x
        Dxg1N = ( XG1NPAR(x+h/2.D0,Q2) - XG1NPAR(x-h/2.D0,Q2) ) / h
C...Second derivative wrt x
        DDxg1N = ( XG1NPAR(x+h,Q2) - 2.D0*XG1NPAR(x,Q2)
     &           + XG1NPAR(x-h,Q2) ) / h**2
C...Derivative wrt p2
        Dp2xg1N = xg1N * dphi_dp2
     &          + (Dxg1N-xg1N/x) * x*(1.D0-x)**2
     &            / (M**2*(1.D0-x)**2 - s)
c        print *, 'x,t1,t2=',x,xg1N * dphi_dp2,
c     &          (Dxg1N-xg1N/x) * x*(1.D0-x)**2
c     &            / (M**2*(1.D0-x)**2 - s)
c        print *, 'x,Dp2xg1N=',x,Dp2xg1N


c        xg1N = x**0.5 * (1.D0-x)**3.D0
c        Dxg1N = 0.5D0*(1-x)**3/x**0.5 - 3*(1-x)**2*x**0.5
c        DDxg1N = -0.25*(1-x)**3/x**1.5 - 3*(1-x)**2/x**0.5
c     &         + 6*(1-x)*x**0.5

c	print *, 'leaving G1PAR...xg1p,xg1n=',xg1p,xg1n

        RETURN
        END


C **********************************************************************
	FUNCTION XG1NPAR (Nflag,x,Q2)
C
C  Function gives values of x*g1N at specified x & Q2
C  Nflag=1 proton
C  Nflag=0 neutron
C **********************************************************************
	IMPLICIT NONE
	INTEGER	Nflag
        REAL*8  XG1NPAR, x,Q2
        LOGICAL init /.FALSE./
        REAL*8  xg1p,xg1nt,xg1D
        REAL*8  pi,alphas,LQCD,Nf,beta0,
     &          etau,etad,etaG,au,ad,aG,bu,bd,bG,gu,gd,gG,
     &          AAu,AAd,AAG, Duv,Ddv,Dg,q3,q8,q0
        COMPLEX*16 GAMMA

        pi = 4.D0*DATAN(1.D0)

        IF (init) GO TO 12
C...Germann & Stirling parameterisation (problems differentiating)
c        CALL GS_S_INI
c        init = .TRUE.
c 12     CALL GS_G1 (0,x,Q2,xg1p,xg1nt,xg1D)
c        xg1npar = x**0.5D0 * (1.D0-x)**3.d0
c        XG1NPAR = (xg1p + xg1nt)/2.D0
c        XG1NPAR = xg1p
c        IF (.true.) RETURN

C...Analytic input (from Germann & Stirling) at Q2 = 4 GeV^2
C...Uses singlet => singlet + glue decomposition ansatz
C...Dqv = x*Delta qV(x)
        LQCD = 0.177D0
        Nf = 3.D0
        beta0 = 11.D0 - 2.D0/3.D0*Nf
        alphas = 4.D0 * pi * ( 1.D0 / beta0 / LOG(Q2/LQCD**2) )

        etau = 0.848D0
        etad = -0.294D0
        etaG = 1.971D0
        au = 0.46D0
        ad = au
        aG = 1.D0
        bu = 3.64D0
        bd = bu + 1.D0
        bG = 7.44D0
        gu = 18.36D0
        gd = gu
        gG = 0.D0
        AAu = (1.D0 + gu*au/(au+bu+1.D0))
     &      * DREAL(GAMMA(DCMPLX(au)))
     &      * DREAL(GAMMA(DCMPLX(bu+1)))
     &      / DREAL(GAMMA(DCMPLX(au+bu+1)))
        AAu = 1.D0/AAu
        AAd = (1.D0 + gd*ad/(ad+bd+1.D0))
     &      * DREAL(GAMMA(DCMPLX(ad)))
     &      * DREAL(GAMMA(DCMPLX(bd+1)))
     &      / DREAL(GAMMA(DCMPLX(ad+bd+1)))
        AAd = 1.D0/AAd
        AAG = (1.D0 + gG*aG/(aG+bG+1.D0))
     &      * DREAL(GAMMA(DCMPLX(aG)))
     &      * DREAL(GAMMA(DCMPLX(bG+1)))
     &      / DREAL(GAMMA(DCMPLX(aG+bG+1)))
        AAg = 1.D0/AAg
        init = .TRUE.

 12     CONTINUE
        Duv = etau * AAu * x**au * (1.D0-x)**bu * (1.D0+gu*x)
        Ddv = etad * AAd * x**ad * (1.D0-x)**bd * (1.D0+gd*x)
        Dg =  etaG * AAG * x**aG * (1.D0-x)**bG * (1.D0+gG*x)

        q3 = Duv-Ddv
        q8 = Duv+Ddv
        q0 = Duv+Ddv
        xg1p  =  q3/12.D0 + q8/36.D0 + q0/9.D0 - Dg*alphas/3.D0/2.D0/pi
        xg1nt = -q3/12.D0 + q8/36.D0 + q0/9.D0 - Dg*alphas/3.D0/2.D0/pi
c	XG1NPAR = (xg1p + xg1nt)/2.D0
	IF (Nflag.EQ.1) XG1NPAR = xg1p
c	IF (Nflag.EQ.0) XG1NPAR = xg1nt
	IF (Nflag.EQ.0) XG1NPAR = 1.09375 * x**0.5 * (1.0-x)**3

        RETURN
        END


C **********************************************************************
        SUBROUTINE G2PAR (x,Q2,xg2p,xg2n)
C
C        SUBROUTINE G2PAR (x,Q2,xg2p,xg2n,Dxg2N,DDxg2N,Dp2xg2N)
C  g2 structure function and derivatives
C **********************************************************************
	IMPLICIT NONE
        REAL*8  x,Q2,xg2p,xg2n,Dxg2N,DDxg2N,Dp2xg2N
        REAL*8  h,XG2NPAR
        REAL*8  s,M,xx,xxmax,xxmin,xxint,g2N,dphi_dp2,fac,n2,nd2
        INTEGER nxx,ixx
c	LOGICAL p2norm /.FALSE./
        LOGICAL p2norm /.true./
        COMMON  /phi/ dphi_dp2


        M = 938.91897D0  ! Nucleon mass (MeV)
        s = 2.0D0 * 1.D6 ! Mass squared of spectator system (MeV^2)

        IF (p2norm) GO TO 172
C...Check normalisation of dg2/dp2
        xxmax = 1.D0
        xxmin = 0.D0
        nxx = 10000
        xxint = (xxmax-xxmin)/DFLOAT(nxx)

        n2 = 0.D0
        nd2 = 0.D0
        ixx = 1
        DO xx=xxint,xxmax-xxint,xxint
          g2N = XG2NPAR(xx,Q2)/xx
          fac = g2N * (1.D0-xx)
     &         * ( (3.D0*xx - 1.D0)*s + M**2*(1.D0-xx)**3 )
     &         / (M**2*(1.D0-xx)**2 - s)**2
          IF (ixx.EQ.0 .OR. ixx.EQ.nxx) THEN
            n2  = n2  + g2N
            nd2 = nd2 + fac
          ELSE IF (ixx/2*2.NE.ixx) THEN
            n2  = n2  + 4.D0*g2N
            nd2 = nd2 + 4.D0*fac
          ELSE IF (ixx/2*2.EQ.ixx) THEN
            n2  = n2  + 2.D0*g2N
            nd2 = nd2 + 2.D0*fac
          ENDIF
          if(ixx/(nxx/100)*(nxx/100).eq.ixx)
     &      print *,'G2:xx,g2,fac=',xx,g2N,fac
c        print *,'G2:xx,g2,fac=',xx,g2N,fac
          ixx = ixx + 1
        ENDDO
        n2 = (xxint/3.D0) * n2
        nd2 = (xxint/3.D0) * nd2
        print *, 'norm(g2),norm(dg2/dp,GeV-2)=',n2,nd2*1.D6
        p2norm = .TRUE.


 172    h = 0.001D0
C...Value of x*g2N
        xg2p = XG2NPAR(1,x,Q2)
        xg2n = XG2NPAR(0,x,Q2)
C...First derivative wrt x
        Dxg2N = ( XG2NPAR(x+h/2.D0,Q2) - XG2NPAR(x-h/2.D0,Q2) ) / h
C...Second derivative wrt x
        DDxg2N = ( XG2NPAR(x+h,Q2) - 2.D0*XG2NPAR(x,Q2)
     &           + XG2NPAR(x-h,Q2) ) / h**2
C...Derivative wrt p2
        Dp2xg2N = xg2N * dphi_dp2
     &          + (Dxg2N-xg2N/x) * x*(1.D0-x)**2
     &            / (M**2*(1.D0-x)**2 - s)

c        xg2N = x**0.5 * (1.D0-x)**3.D0
c        Dxg2N = 0.5D0*(1-x)**3/x**0.5 - 3*(1-x)**2*x**0.5
c        DDxg2N = -0.25*(1-x)**3/x**1.5 - 3*(1-x)**2/x**0.5
c     &         + 6*(1-x)*x**0.5

        RETURN
        END


C **********************************************************************
        FUNCTION XG2NPAR (Nflag,x,Q2)
C  Function gives values of x*g2N at specified x & Q2
C  Nflag = 1 proton
C  Nflag = 0 neutron
C **********************************************************************
	IMPLICIT NONE
        INTEGER Nflag,iy
        REAL*8  XG2NPAR, x,Q2
        REAL*8  y,yint,s0,XG1NPAR,tw3

C...Twist-2 Wandura-Wilczek piece
        XG2NPAR = 0.D0
c	if(.true.)goto 284
        yint = (1.D0-x)/1000
        iy = 0
        DO y = x,1.D0,yint

          s0 = x * XG1NPAR(Nflag,y,Q2) / y**2

          IF (iy.EQ.0) THEN
            XG2NPAR = XG2NPAR + s0
          ELSE IF (iy/2*2.NE.iy) THEN
            XG2NPAR = XG2NPAR + 4.D0*s0
          ELSE IF (iy/2*2.EQ.iy) THEN
            XG2NPAR = XG2NPAR + 2.D0*s0
          ENDIF
          iy = iy + 1
c        print *, 'y,s0=',y,s0
        ENDDO
        XG2NPAR = (yint/3.D0) * XG2NPAR
        XG2NPAR = XG2NPAR - XG1NPAR(x,Q2)

 284    continue
C...Twist-3 piece
C...Fit curve in Fig.6 of Stratmann (Q^2 ~ 1-10 GeV^2)
C...Proton valence only
c        tw3 = -0.07D0 * x**0.3D0 * (1.D0-x)**4 * (1.D0-4*x**0.46D0)
C...Nucleon guess
        tw3 = -0.07D0 * x**0.6D0 * (1.D0-x)**4 * (1.D0-3.623D0*x**0.5D0)
        tw3 = 0.D0
c        print *, 'x,tw3,tot=',x,tw3/x,(xg2npar+tw3)/x

C...Total
        XG2NPAR = XG2NPAR + tw3

        RETURN
        END


C **********************************************************************
        FUNCTION GAMMA (x)
C  Euler Gamma function (Stirling's formula) - from Andreas
C **********************************************************************
	IMPLICIT NONE
        COMPLEX*16 GAMMA,x,GAMMA2
        IF (ABS(x).LT.1D-8) THEN
          WRITE (*,*) 'WARNING---------GAMMA UNDERFLOW--------------'
          GAMMA = x/ABS(x)*1.D245
          RETURN
        ENDIF

        IF (ABS(x).LT.2.D0) THEN
          GAMMA = GAMMA2(x+1.D0)/x
          RETURN
        ENDIF

        IF (ABS(x).GT.143) THEN
        WRITE (*,*) 'WARNING----------GAMMA OVERFLOW---------------'
          GAMMA = 1.D245
          RETURN
        ENDIF

        GAMMA = EXP(-1.D0*x) * x**(x-0.5D0) * SQRT(2.D0*3.1415926535)
     &        * ( 1.D0 + 1.D0/(12.D0*x) + 1.D0/(288.D0*x**2)
     &           - 139.D0/(51840.D0*x**3) - 571.D0/(2488320.D0*x**4) )
        RETURN
        END


C **********************************************************************
        FUNCTION GAMMA2(x)
C **********************************************************************
	IMPLICIT NONE
        COMPLEX*16 GAMMA2,x,GAMMA

        IF (ABS(x).LT.2.D0) THEN
          GAMMA2 = GAMMA(x+1.0)/x
          RETURN
        ENDIF

        GAMMA2 = EXP(-1.D0*x) * x**(x-0.5D0)
     &         * SQRT(2.D0*3.1415926535D0)
     &         * (1.D0 + 1.D0/(12.D0*x) + 1.D0/(288.D0*x**2)
     &            - 139.D0/(51840.D0*x**3) - 571.D0/(2488320.D0*x**4) )
        RETURN
        END
