C *******************************************************************
	PROGRAM HEGAUSS
C
C     TITEL:       SUMREG (-> HE)
C     VERSION:     11.MAI.1994                                      *
C     REVISED:                                                      *
C                                                                   *
C     SUBROUTINES:  nag-routines                                    *
C                                                                   *
C     Description:  This routine calculates the two dimensional     *
C                   integral over the spectral function             *
C                   \int d3p_N dE S(p_N,E)
C               In addition:                                        *
C               This routine is intended to give precise hints on   *
C               how the spectral function is normalized and saved.  *
C               The spectral function is based on the three-body    *
C               wave function calculated on a finite grid with      *
C               40*30 mesh points for the (p,q=p_N)-momenta. It is  *
C               again calculated on a finite grid with (30*30) mesh *
C               points for the momentum/energy mesh (p_N,E).        *
C                                                                   * 
C 	Energy in MeV
C	Momentum in 1/fm
C                                                                   *
C     Compile with: %f77 -r8 sumreg.f                               * 
C                   %f77 -o prog sumreg.o -lnag                     *
C                                                                   *
C
C  Modified for JLab: March 03
C
C  With IMSL routines:
C  compile > use imsl
C          > f77 -o objectfile file.f
C                -R/site/vni/lib/lib.solaris
C                -L/site/vni/lib/lib.solaris -limsl -lsocket -lnsl
C
C *******************************************************************
	IMPLICIT NONE
	INTEGER	NDE,NDE1,NDP,NDQ,NERG,IE,NQ,IQ,JQ,I,J, IEN,NERG1,
     &		neng,ieng,ip,np
	PARAMETER (NDE=32,NDE1=NDE-1,NDP=40,NDQ=30)
	REAL*4	ENER(NDE),QMESH(NDQ),QWEIGH(NDQ),
     &		R0T0(NDP,NDQ),R0T1(NDP,NDQ),
     &		R1T0(NDP,NDQ),R1T1(NDP,NDQ),
     &		R2T0(NDP,NDQ),R2T1(NDP,NDQ)
	REAL*4	F0T0,F0T1,F1T0,F1T1,F2T0,F2T1, F1T0p2,F1T1p2
	REAL*4	ANS,Earray(29),Farray(29),Parray(30),yy,QDVAL
	REAL*4	eng,engmax,engint, p,pmax,pint
	REAL*4	N0p,N0n,N1p,N1n,N2p,N2n, T1p,T1n
	REAL*4	mN,hc,pi

	hc = 193.0	! MeV.fm
	mN = 939.0/hc	! 1/fm
	PI=4.0*ATAN(1.0)

	OPEN(UNIT=1,FILE='spfn.dat')
c	OPEN(UNIT=10,FILE='specout.dat')


C-----input of the spectral function:
C
C     The spectral function is given for the pair-isospin 
C     T=0 and T=1
C     The two-body breakup is given in the first columne, i.e.,
C     R0T0(1,J) etc.
C     R0T0(2..NERG,J) contains the three-body breakup.
C
C     The normalization is chosen such that the proton and 
C     neutron contribution is given by:
C
C      R0P = (F0T0 + F0T1)*4.*PI/3.
C      R0N = 2.*F0T1*4.*PI/3.
C
C     with R0P = 2/3
C     and  R0N = 1/3
C
C     Since the spectral function is given on a finite mesh
C     the sumrule is not exactly fullfilled !
C     The mesh was chosen for a particular kinematic situation.
C
C     The momentum mesh is given for Gaussian points.  


	READ(1,'(2I3)')NERG,NQ
	READ(1,'(6E13.6)')(ENER(IE),IE=1,NERG)
	READ(1,'(6E13.6)')(QMESH(IQ),IQ=1,NQ),(QWEIGH(IQ),IQ=1,NQ)
	READ(1,'(6E13.6)')((R0T0(I,J),R1T0(I,J),R2T0(I,J),
     *			   J=1,NQ),I=1,NERG)
	READ(1,'(6E13.6)')((R0T1(I,J),R1T1(I,J),R2T1(I,J),
     *			   J=1,NQ),I=1,NERG)
	CLOSE (1)

	F0T0 = 0.0
	F1T0 = 0.0
	F2T0 = 0.0
	F0T1 = 0.0
	F1T1 = 0.0
	F2T1 = 0.0

C...Set up energy array (MeV)
	DO IEN = 2,NERG
	  Earray(IEN-1) = ENER(IEN)
	ENDDO	
	engmax = 150.0
	neng = 1000
	engint = engmax/neng
	NERG1 = NERG - 1


C...Summation over all momenta (1/fm)
	pmax = 25.0
	np = 1000
	pint = pmax/np
	ip = 1
c	DO p=0,pmax,pint
	DO JQ=1,NQ

C...f0T0
	  DO IEN=2,NERG
	    Farray(IEN-1) = R0T0(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F0T0 = F0T0 + (ANS + R0T0(1,JQ))*QMESH(JQ)**2*QWEIGH(JQ)
	print *, 'JQ,QMESH,ANS+R0T0=',JQ,QMESH(JQ),ANS+R0T0(1,jq)

C...f0T1
	  DO IEN=2,NERG
	    Farray(IEN-1) = R0T1(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F0T1 = F0T1 + ANS*QMESH(JQ)**2*QWEIGH(JQ)

C...f1T0
	  DO IEN=2,NERG
	    Farray(IEN-1) = R1T0(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F1T0 = F1T0 + (ANS + R1T0(1,JQ))*QMESH(JQ)**2*QWEIGH(JQ)

C...f1T1
	  DO IEN=2,NERG
	    Farray(IEN-1) = R1T1(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F1T1 = F1T1 + ANS*QMESH(JQ)**2*QWEIGH(JQ)

C...f2T0
	  DO IEN=2,NERG
	    Farray(IEN-1) = R2T0(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F2T0 = F2T0 + (ANS + R2T0(1,JQ))*QMESH(JQ)**2*QWEIGH(JQ)

C...f2T1
	  DO IEN=2,NERG
	    Farray(IEN-1) = R2T1(IEN,JQ)
	  ENDDO

	  ieng = 0
	  ANS = 0.0			! energy integration
	  DO eng=0.0,engmax,engint
	    yy = QDVAL (eng,NERG1,Earray,Farray,.FALSE.)
	    IF (ieng.EQ.0)        ANS = ANS + yy
	    IF (ieng/2*2.NE.ieng) ANS = ANS + 4*yy
	    IF (ieng/2*2.EQ.ieng) ANS = ANS + 2*yy
	    ieng = ieng + 1
	  ENDDO
	  ANS = (engint/3)*ANS

	  F2T1 = F2T1 + ANS*QMESH(JQ)**2*QWEIGH(JQ)

	ENDDO

	print *, '*** F0T0 = ',f0t0

	N0p = (4.*PI) * (F0T0 + F0T1)
	N0n = (4.*PI) * 2.*F0T1
	N1p = (4.*PI) * (F1T0 + F1T1)
	N1n = (4.*PI) * 2.*F1T1
	N2p = (4.*PI) * (F2T0 + F2T1)
	N2n = (4.*PI) * 2.*F2T1

	print *, '--------------------------------------'
	print *, 'N0p,N0n=',n0p,n0n
	print *
	print *, 'N1p,N1n=',n1p,n1n
	print *
	print *, '(N1p+N0p)/2,(N1n+N0n)/2=',(n0p+n1p)/2,(n0n+n1n)/2
	print *
	print *, 'N2p,N2n=',n2p,n2n
	print *

	STOP
	END
